<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * BABE_Post_types Class.
 * 
 * @class 		BABE_Post_types
 * @version		1.1.1
 * @author 		Booking Algorithms
 */

class BABE_Post_types {
    
    // variables to use
    static $booking_obj_post_type = 'to_book';
    static $faq_post_type = 'faq';
    static $service_post_type = 'service';
    static $order_post_type = 'order';
    static $mpoints_post_type = 'places';
    static $coupon_post_type = 'coupon';
    
    public static $search_filter_sort_by_args = array();
    
    // service type names
    static $service_type_names = array();
    
    static $categories_tax = 'categories';
    static $ages_tax = 'ages';
    static $taxonomies_list;  // array of custom taxonomies names and slugs
    // will store in this taxonomy all new custom taxonomies for booking objects 
    static $taxonomies_list_tax = 'taxonomies_list';
    
    static $attr_tax_pref = 'ba_';
    
    static $get_posts_count = 0;
    static $get_posts_pages = 0;
    
    //// cache
    
    private static $ages_arr = array();
    
    private static $categories_arr = array();
    
    private static $post = array();
    
    private static $post_discount = array();
    
    private static $post_meta = array();
    
//////////////////////////////
    /**
	 * Hook in tabs
	 */
    public static function init() {
		add_action( 'init', array( __CLASS__, 'register_post_types' ), 0 );
        add_action( 'init', array( __CLASS__, 'register_taxonomies' ), 0 );
        add_action( 'init', array( __CLASS__, 'register_taxonomies_list' ), 0 );
        add_filter('babe_sanitize_'.BABE_Settings::$option_name, array( __CLASS__, 'update_booking_obj_posts_slug'), 10);
        
        add_filter('babe_sanitize_'.BABE_Settings::$option_name, array( __CLASS__, 'update_mpoints'), 10);
        
        self::$service_type_names = self::get_service_type_names();
        
        self::$search_filter_sort_by_args = self::get_search_filter_sort_by_args();  
	}
    
///////////////////////////////////////
    /**
	 * Get service type names.
     * @return array
	 */
    public static function get_service_type_names(){ 
    
    return array(
            'booking' => __( 'per Booking', BABE_TEXTDOMAIN ),
            'person' => __( 'per Person', BABE_TEXTDOMAIN ),
            'day' => __( 'per Day', BABE_TEXTDOMAIN ),
            'night' => __( 'per Night', BABE_TEXTDOMAIN ),
            'person_day' => __( 'per Person/day', BABE_TEXTDOMAIN ),
            'person_night' => __( 'per Person/night', BABE_TEXTDOMAIN ),
         );          
    }
    
///////////////////////////////////////
    /**
	 * Get sort by args for serach results filter.
     * @return array
	 */
    public static function get_search_filter_sort_by_args(){
    
    return apply_filters('babe_search_filter_sort_by_args', array(
              'price_from_asc' => __( 'Price from low', BABE_TEXTDOMAIN ),
              'price_from_desc' => __( 'Price from high', BABE_TEXTDOMAIN ),
              'rating' => __( 'Rating', BABE_TEXTDOMAIN ),
            ));          
    }
    
///////////////////////////////////////
    /**
	 * Translate search form args to get posts args.
     * @param array $args - search form args
     * @return array
	 */
    public static function search_filter_to_get_posts_args($args = array()){
    
       if (isset($args['search_results_sort_by'])){
          switch ($args['search_results_sort_by']){
                case 'price_from_desc':
                   $args['sort'] = 'price_from';
                   $args['sort_by'] = 'DESC';
                   break;
                case 'rating':
                   $args['sort'] = 'rating';
                   $args['sort_by'] = 'DESC';
                   break;
                case 'price_from_asc':
                default:
                   $args['sort'] = 'price_from';
                   $args['sort_by'] = 'ASC';
                   $args['search_results_sort_by'] = 'price_from_asc';
                   break;      
          }
        }
            
        return $args;              
    }            
    
//////////////////////////////
    /**
	 * Register post types.
	 */
    public static function register_post_types() {
        
    $labels = array(
		'name'               => BABE_Settings::$settings['booking_obj_post_name_general'],
		'singular_name'      => BABE_Settings::$settings['booking_obj_post_name'],
		'menu_name'          => BABE_Settings::$settings['booking_obj_menu_name'],
		'name_admin_bar'     => BABE_Settings::$settings['booking_obj_post_name'],
		'add_new'            => __( 'Add ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'add_new_item'       => __( 'Add New ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'new_item'           => __( 'New ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'edit_item'          => __( 'Edit ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'view_item'          => __( 'View ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'all_items'          => __( 'All ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name_general'],
		'search_items'       => __( 'Search ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'],
		'parent_item_colon'  => __( 'Parent ', BABE_TEXTDOMAIN ).BABE_Settings::$settings['booking_obj_post_name'].':',
		'not_found'          => sprintf(__( 'No %s found.', BABE_TEXTDOMAIN ), BABE_Settings::$settings['booking_obj_post_name_general']),
		'not_found_in_trash' => sprintf(__( 'No %s found in Trash.', BABE_TEXTDOMAIN ), BABE_Settings::$settings['booking_obj_post_name_general'])
	);    

// Set other options for Custom Post Type

	$args = array(
		'description'         => BABE_Settings::$settings['booking_obj_post_name_general'],
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'title', 'editor', 'thumbnail', 'page-attributes', 'excerpt', 'comments' ),
		'hierarchical'        => true,
		'public'              => true,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 33,
        'menu_icon'           => 'dashicons-clipboard',
		'show_in_nav_menus'   => true,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => false,
		'publicly_queryable'  => true,
		'capability_type'     => 'post',
        'rewrite' => array(
                'slug' => BABE_Settings::$settings['booking_obj_post_slug']
            ),       
	);
    
    if (BABE_Settings::$settings['booking_obj_gutenberg']){
        $args['show_in_rest'] = true;
    }

	// Registering our Post Type
	register_post_type( self::$booking_obj_post_type, $args );

    $labels = array(
		'name'                => _x( 'FAQ', 'Post Type General Name', BABE_TEXTDOMAIN ),
		'singular_name'       => _x( 'FAQ', 'Post Type Singular Name', BABE_TEXTDOMAIN ),
        'menu_name'          => _x( 'FAQ', 'admin menu', BABE_TEXTDOMAIN ),
		'name_admin_bar'     => _x( 'FAQ', 'add new on admin bar', BABE_TEXTDOMAIN ),
		'add_new'            => _x( 'Add FAQ', 'booking object', BABE_TEXTDOMAIN ),
        'add_new_item'        => __( 'Add New FAQ', BABE_TEXTDOMAIN ),
        'new_item'           => __( 'New FAQ', BABE_TEXTDOMAIN ),
		'edit_item'          => __( 'Edit FAQ', BABE_TEXTDOMAIN ),
		'view_item'          => __( 'View FAQ', BABE_TEXTDOMAIN ),
		'all_items'          => __( 'All FAQs', BABE_TEXTDOMAIN ),
		'search_items'       => __( 'Search FAQ', BABE_TEXTDOMAIN ),
		'parent_item_colon'  => __( 'Parent FAQ:', BABE_TEXTDOMAIN ),
		'not_found'          => __( 'Not found.', BABE_TEXTDOMAIN ),
		'not_found_in_trash' => __( 'Not found in Trash.', BABE_TEXTDOMAIN ),
		'update_item'         => __( 'Update FAQ', BABE_TEXTDOMAIN ),
	);

// Set other options for Custom Post Type

	$args = array(
		'description'         => __( 'FAQ', BABE_TEXTDOMAIN ),
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'title', 'editor', 'page-attributes' ),
		'hierarchical'        => true,
		'public'              => false,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 34,
		'show_in_nav_menus'   => false,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => true,
		'publicly_queryable'  => true,
		'capability_type'     => 'post',
	);

	// Registering your Custom Post Type
	register_post_type( self::$faq_post_type, $args );
    
    $labels = array(
		'name'                => _x( 'Services', 'Post Type General Name', BABE_TEXTDOMAIN ),
		'singular_name'       => _x( 'Service', 'Post Type Singular Name', BABE_TEXTDOMAIN ),
        'menu_name'          => _x( 'Services', 'admin menu', BABE_TEXTDOMAIN ),
		'name_admin_bar'     => _x( 'Services', 'add new on admin bar', BABE_TEXTDOMAIN ),
		'add_new'            => _x( 'Add Service', 'booking object', BABE_TEXTDOMAIN ),
        'add_new_item'        => __( 'Add New Service', BABE_TEXTDOMAIN ),
        'new_item'           => __( 'New Service', BABE_TEXTDOMAIN ),
		'edit_item'          => __( 'Edit Service', BABE_TEXTDOMAIN ),
		'view_item'          => __( 'View Service', BABE_TEXTDOMAIN ),
		'all_items'          => __( 'All Services', BABE_TEXTDOMAIN ),
		'search_items'       => __( 'Search Service', BABE_TEXTDOMAIN ),
		'parent_item_colon'  => __( 'Parent Service:', BABE_TEXTDOMAIN ),
		'not_found'          => __( 'Not found.', BABE_TEXTDOMAIN ),
		'not_found_in_trash' => __( 'Not found in Trash.', BABE_TEXTDOMAIN ),
		'update_item'         => __( 'Update Service', BABE_TEXTDOMAIN ),
	);

// Set other options for Custom Post Type

	$args = array(
		'description'         => __( 'Services', BABE_TEXTDOMAIN ),
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'title', 'editor', 'thumbnail', 'page-attributes' ),
		'hierarchical'        => true,
		'public'              => false,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 35,
		'show_in_nav_menus'   => false,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => true,
		'exclude_from_search' => true,
		'publicly_queryable'  => true,
		'capability_type'     => 'post',
        'rewrite' => array(
                'slug' => 'services',
            ),
        'show_in_rest' => true,
	);

	// Registering your Custom Post Type
	register_post_type( self::$service_post_type, $args );
    
    $labels = array(
		'name'                => _x( 'Orders', 'Post Type General Name', BABE_TEXTDOMAIN ),
		'singular_name'       => _x( 'Order', 'Post Type Singular Name', BABE_TEXTDOMAIN ),
        'menu_name'          => _x( 'Orders', 'admin menu', BABE_TEXTDOMAIN ),
		'name_admin_bar'     => _x( 'Orders', 'add new on admin bar', BABE_TEXTDOMAIN ),
		'add_new'            => _x( 'Add Order', 'booking object', BABE_TEXTDOMAIN ),
        'add_new_item'        => __( 'Add New Order', BABE_TEXTDOMAIN ),
        'new_item'           => __( 'New Order', BABE_TEXTDOMAIN ),
		'edit_item'          => __( 'Edit Order', BABE_TEXTDOMAIN ),
		'view_item'          => __( 'View Order', BABE_TEXTDOMAIN ),
		'all_items'          => __( 'All Orders', BABE_TEXTDOMAIN ),
		'search_items'       => __( 'Search Order', BABE_TEXTDOMAIN ),
		'parent_item_colon'  => __( 'Parent Order:', BABE_TEXTDOMAIN ),
		'not_found'          => __( 'Not found.', BABE_TEXTDOMAIN ),
		'not_found_in_trash' => __( 'Not found in Trash.', BABE_TEXTDOMAIN ),
		'update_item'         => __( 'Update Order', BABE_TEXTDOMAIN ),
	);

// Set other options for Custom Post Type

	$args = array(
		'description'         => __( 'Order', BABE_TEXTDOMAIN ),
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'love' ),
		'hierarchical'        => false,
		'public'              => false,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 35,
		'show_in_nav_menus'   => false,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => true,
		'publicly_queryable'  => false,
		'capability_type'     => 'post',
	);
    
	register_post_type( self::$order_post_type, $args );
    ////////////////////////
    
    if (BABE_Coupons::coupons_active()){
    
    $labels = array(
		'name'                => _x( 'Coupons', 'Post Type General Name', BABE_TEXTDOMAIN ),
		'singular_name'       => _x( 'Coupon', 'Post Type Singular Name', BABE_TEXTDOMAIN ),
        'menu_name'          => _x( 'Coupons', 'admin menu', BABE_TEXTDOMAIN ),
		'name_admin_bar'     => _x( 'Coupon', 'add new on admin bar', BABE_TEXTDOMAIN ),
		'add_new'            => _x( 'Add Coupon', 'booking object', BABE_TEXTDOMAIN ),
        'add_new_item'        => __( 'Add New Coupon', BABE_TEXTDOMAIN ),
        'new_item'           => __( 'New Coupon', BABE_TEXTDOMAIN ),
		'edit_item'          => __( 'Edit Coupon', BABE_TEXTDOMAIN ),
		'view_item'          => __( 'View Coupon', BABE_TEXTDOMAIN ),
		'all_items'          => __( 'All Coupons', BABE_TEXTDOMAIN ),
		'search_items'       => __( 'Search Coupon', BABE_TEXTDOMAIN ),
		'parent_item_colon'  => __( 'Parent Coupon:', BABE_TEXTDOMAIN ),
		'not_found'          => __( 'Not found.', BABE_TEXTDOMAIN ),
		'not_found_in_trash' => __( 'Not found in Trash.', BABE_TEXTDOMAIN ),
		'update_item'         => __( 'Update Coupon', BABE_TEXTDOMAIN ),
	);

// Set other options for Custom Post Type

	$args = array(
		'description'         => __( 'Coupon', BABE_TEXTDOMAIN ),
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'love' ),
		'hierarchical'        => true,
		'public'              => false,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 34,
		'show_in_nav_menus'   => false,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => true,
		'publicly_queryable'  => false,
		'capability_type'     => 'post',
	);

	// Registering your Custom Post Type
	register_post_type( self::$coupon_post_type, $args );
    
    }
    
    ////////////////////////////
    
    if (BABE_Settings::$settings['mpoints_active']){
        
        $labels = array(
		'name'                => _x( 'Places', 'Post Type General Name', BABE_TEXTDOMAIN ),
		'singular_name'       => _x( 'Place', 'Post Type Singular Name', BABE_TEXTDOMAIN ),
        'menu_name'          => _x( 'Places', 'admin menu', BABE_TEXTDOMAIN ),
		'name_admin_bar'     => _x( 'Places', 'add new on admin bar', BABE_TEXTDOMAIN ),
		'add_new'            => _x( 'Add Place', 'booking object', BABE_TEXTDOMAIN ),
        'add_new_item'        => __( 'Add New Place', BABE_TEXTDOMAIN ),
        'new_item'           => __( 'New Place', BABE_TEXTDOMAIN ),
		'edit_item'          => __( 'Edit Place', BABE_TEXTDOMAIN ),
		'view_item'          => __( 'View Place', BABE_TEXTDOMAIN ),
		'all_items'          => __( 'All Places', BABE_TEXTDOMAIN ),
		'search_items'       => __( 'Search Place', BABE_TEXTDOMAIN ),
		'parent_item_colon'  => __( 'Parent Place:', BABE_TEXTDOMAIN ),
		'not_found'          => __( 'Not found.', BABE_TEXTDOMAIN ),
		'not_found_in_trash' => __( 'Not found in Trash.', BABE_TEXTDOMAIN ),
		'update_item'         => __( 'Update Place', BABE_TEXTDOMAIN ),
	);

// Set other options for Custom Post Type

	$args = array(
		'description'         => __( 'Place', BABE_TEXTDOMAIN ),
		'labels'              => $labels,
		// Features this CPT supports
		'supports'            => array( 'title', 'editor', 'thumbnail', 'page-attributes' ),
		'hierarchical'        => true,
		'public'              => true,
		'show_ui'             => true,
		'show_in_menu'        => true,
        'menu_position'        => 35,
		'show_in_nav_menus'   => true,
		'show_in_admin_bar'   => true,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => true,
		'publicly_queryable'  => true,
		'capability_type'     => 'post',
        'show_in_rest' => true,
	);

	// Registering your Custom Post Type
	register_post_type( self::$mpoints_post_type, $args );
        
    } ////////////////
    
    if (get_option('_babe_posts_need_rewrite_rules')){
        update_option('_babe_posts_need_rewrite_rules', 0);
        flush_rewrite_rules();
    }
    
    }    
    
/////////////////////////////
     /**
	 * Change booking_obj post slug for all posts in DB.
     * @param array $new_input
     * @return array
	 */ 
     public static function update_booking_obj_posts_slug($new_input){ 
         if (BABE_Settings::$settings['booking_obj_post_slug'] != $new_input['booking_obj_post_slug']){
            update_option('_babe_posts_need_rewrite_rules', 1);
         }   
        
        return $new_input;
    }
    
/////////////////////////////
     /**
	 * Change booking_obj post slug for all posts in DB.
     * @param array $new_input
     * @return array
	 */ 
     public static function update_mpoints($new_input){ 
         if (BABE_Settings::$settings['mpoints_active'] != $new_input['mpoints_active']){
            update_option('_babe_posts_need_rewrite_rules', 1);
         }   
        
        return $new_input;
    }        
    
//////////////////////////////
    /**
	 * Register taxonomies.
	 */
    public static function register_taxonomies() {
        
       //// categories 
        
       $labels = array(
			'name'              => __( 'Categories', BABE_TEXTDOMAIN ),
			'singular_name'     => __( 'Category', BABE_TEXTDOMAIN ),
			'search_items'      => __( 'Search Categories', BABE_TEXTDOMAIN ),
			'all_items'         => __( 'All Categories', BABE_TEXTDOMAIN ),
			'parent_item'       => __( 'Parent Category', BABE_TEXTDOMAIN ),
			'parent_item_colon' => __( 'Parent Category:', BABE_TEXTDOMAIN ),
			'edit_item'         => __( 'Edit Category', BABE_TEXTDOMAIN ),
			'update_itm'        => __( 'Update Category', BABE_TEXTDOMAIN ),
			'add_new_item'      => __( 'Add New Category', BABE_TEXTDOMAIN ),
			'new_item_name'     => __( 'New Category', BABE_TEXTDOMAIN ),
			'menu_name'         => __( 'Categories', BABE_TEXTDOMAIN ),
		);

		register_taxonomy( self::$categories_tax, self::$booking_obj_post_type, array(
			'labels'            => $labels,
			'hierarchical'      => false,
			'query_var'         => self::$categories_tax,
			'public'            => true,
			'show_ui'           => true,
            'show_in_nav_menus'   => false,
			'show_admin_column' => true,
		) );
        
        //// list_taxonomy
        
        $labels = array(
			'name'              => __( 'Taxonomies', BABE_TEXTDOMAIN ),
			'singular_name'     => __( 'Taxonomy', BABE_TEXTDOMAIN ),
			'search_items'      => __( 'Search Taxonomies', BABE_TEXTDOMAIN ),
			'all_items'         => __( 'All Taxonomies', BABE_TEXTDOMAIN ),
			'parent_item'       => __( 'Parent Taxonomy', BABE_TEXTDOMAIN ),
			'parent_item_colon' => __( 'Parent Taxonomy:', BABE_TEXTDOMAIN ),
			'edit_item'         => __( 'Edit Taxonomy', BABE_TEXTDOMAIN ),
			'update_itm'        => __( 'Update Taxonomy', BABE_TEXTDOMAIN ),
			'add_new_item'      => __( 'Add New Taxonomy', BABE_TEXTDOMAIN ),
			'new_item_name'     => __( 'New Taxonomy', BABE_TEXTDOMAIN ),
			'menu_name'         => __( 'Taxonomies', BABE_TEXTDOMAIN ),
		);

		register_taxonomy( self::$taxonomies_list_tax, self::$booking_obj_post_type, array(
			'labels'            => $labels,
			'hierarchical'      => false,
			'query_var'         => self::$taxonomies_list_tax,
			'public'            => false,
			'show_ui'           => true,
            'show_in_nav_menus'   => false,
			'show_admin_column' => false,
		) );
        
        self::init_taxonomies_list();
        
        //// ages 
        
       $labels = array(
			'name'              => __( 'Ages', BABE_TEXTDOMAIN ),
			'singular_name'     => __( 'Age', BABE_TEXTDOMAIN ),
			'search_items'      => __( 'Search Ages', BABE_TEXTDOMAIN ),
			'all_items'         => __( 'All Ages', BABE_TEXTDOMAIN ),
			'parent_item'       => __( 'Parent Age', BABE_TEXTDOMAIN ),
			'parent_item_colon' => __( 'Parent Age:', BABE_TEXTDOMAIN ),
			'edit_item'         => __( 'Edit Age', BABE_TEXTDOMAIN ),
			'update_itm'        => __( 'Update Age', BABE_TEXTDOMAIN ),
			'add_new_item'      => __( 'Add New Age', BABE_TEXTDOMAIN ),
			'new_item_name'     => __( 'New Age', BABE_TEXTDOMAIN ),
			'menu_name'         => __( 'Ages', BABE_TEXTDOMAIN ),
		);

		register_taxonomy( self::$ages_tax, self::$booking_obj_post_type, array(
			'labels'            => $labels,
			'hierarchical'      => false,
			'query_var'         => self::$ages_tax,
			'public'            => true,
			'show_ui'           => true,
            'show_in_nav_menus'   => false,
			'show_admin_column' => false,
		) );
         
    }
//////////////////////////////
    /**
	 * Register custom taxonomies based on Taxonomies list terms
	 */
    public static function register_taxonomies_list() {
        
        if ( !empty(self::$taxonomies_list) ){
            foreach( self::$taxonomies_list as $taxonomy ) {
               $new_tax_slug = $taxonomy['slug'];
               $new_tax_name = $taxonomy['name'];
               // create new taxonomy
        
       $labels = array(
			'name'              => $new_tax_name,
			'singular_name'     => $new_tax_name,
			'search_items'      => sprintf(__( 'Search %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'all_items'         => sprintf(__( 'All %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'parent_item'       => sprintf(__( 'Parent %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'parent_item_colon' => sprintf(__( 'Parent %s:', BABE_TEXTDOMAIN ), $new_tax_name),
			'edit_item'         => sprintf(__( 'Edit %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'update_itm'        => sprintf(__( 'Update %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'add_new_item'      => sprintf(__( 'Add New %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'new_item_name'     => sprintf(__( 'New %s', BABE_TEXTDOMAIN ), $new_tax_name),
			'menu_name'         => sprintf(__( '%s', BABE_TEXTDOMAIN ), $new_tax_name),
		);

		register_taxonomy( $new_tax_slug, self::$booking_obj_post_type, array(
			'labels'            => $labels,
			'hierarchical'      => true,
			'query_var'         => $new_tax_slug,
			'public'            => true,
			'show_ui'           => true,
            'show_in_nav_menus'   => false,
			'show_admin_column' => false,
            'show_in_menu' => true,
            'show_in_rest' => true,
		) );
            }
        }
         
    }
       
//////////////////////////////
    /**
	 * Init taxonomies list array.
	 */
    static function init_taxonomies_list() {
        $taxonomies = get_terms( array(
          'taxonomy' => self::$taxonomies_list_tax,
          'hide_empty' => false
        ) );
        
        if ( !empty($taxonomies) ){
            foreach( $taxonomies as $tax_term ) {
                
               self::$taxonomies_list[$tax_term->term_id] = array(
                   'name' => apply_filters('translate_text', $tax_term->name),
                   'slug' => self::$attr_tax_pref.$tax_term->slug,
               ); 
                
               $term_vals = get_term_meta($tax_term->term_id);
               foreach($term_vals as $key=>$val){
                self::$taxonomies_list[$tax_term->term_id][$key] = $val[0];
               }   
            }
        } else {
            self::$taxonomies_list = array();
        }      
    }
////////////////////////////
    /**
	 * Get taxonomy meta by custom taxonomy slug.
     * @param $slug - Custom taxonomy slug
     * @return array
	 */
    static function get_taxonomy_meta_by_slug($slug) {
          $term_id = 0;
          foreach( self::$taxonomies_list as $id => $term_arr ){
              if ($term_arr['slug'] == $slug){
                 $term_id = $id;
                 break;
              }
          }
          $output = $term_id ? self::get_term_meta($term_id) : array();
          return $output;    
    }
////////////////////////////
    /**
	 * Get categories array.
     * @return array
	 */
    public static function get_categories_arr() {
        $output = array();
        
        if (!empty(self::$categories_arr)){
            return self::$categories_arr;  
        }
        
        $taxonomies = get_terms( array(
          'taxonomy' => self::$categories_tax,
          'hide_empty' => false
        ) );
        
        if ( !empty($taxonomies) ){
            foreach( $taxonomies as $tax_term ) {
               $output[$tax_term->term_id] = apply_filters('translate_text', $tax_term->name);  
            }
        }
        
        self::$categories_arr = $output;
        
        return $output;       
    }
////////////////////////////
    /**
	 * Get ages array.
     * @return array
	 */
    public static function get_ages_arr() {
        $output = array();
        
        if (!empty(self::$ages_arr)){
            return self::$ages_arr;  
        }
        
        $ages = get_terms( array(
          'taxonomy' => self::$ages_tax,
          'hide_empty' => false,
          'order' => 'ASC',
          'orderby' => 'meta_value_num',
          'meta_key' => 'menu_order',
        ) );
        
        if ( !empty($ages) ){
            foreach( $ages as $age_term ) {
               $output[intval($age_term->term_id)] = array(
                   'name' => apply_filters('translate_text', $age_term->name),
                   'slug' => $age_term->slug,
                   'description' => apply_filters('translate_text', $age_term->description)
               );  
            }
        }
        
        self::$ages_arr = $output;
        
        return $output;       
    }
    
////////////////////////////
    /**
	 * Clear cached ages
     * 
     * @return
	 */
    public static function clear_cache_ages() {
        
        self::$ages_arr = array();
        
        return;       
    }    
    
////////////////////////////
    /**
	 * Get ages array.
     * @return array
	 */
    public static function get_post_ages($booking_obj_id) {
        $output = array();
        
        $base_rate = BABE_Prices::get_base_rate($booking_obj_id);
        
        if(!empty($base_rate)){      
            $ages = self::get_ages_arr();
            foreach( $ages as $age_id => $age_term ){
                if (isset($base_rate['base'][$age_id])){
                    $output[$age_id] = $age_term;
                }
            }
        }
        
        return $output;
    }        
    
////////////////////////////
    /**
	 * Get main age id by rule.
     * @param array $rule
     * @return int
	 */
    public static function get_main_age_id($rule = array()) {
        $output = 0;

         if ((isset($rule['ages']) && $rule['ages']) || empty($rule)){
              $ages = self::get_ages_arr();
              foreach ($ages as $age_id => $age_arr){
                $output = $age_id;
                break;
              }
         }          
        
        return $output;       
    }    
        
////////////////////////////
    /**
	 * Get category meta.
     * @param $term_id int
     * @return array
	 */
    public static function get_term_meta($term_id) {
        $output = array();
        $term_vals = get_term_meta($term_id);
               foreach($term_vals as $key=>$val){
                $output[$key] = maybe_unserialize($val[0]);
               }
        
        return $output;       
    }
////////////////////////////
    /**
	 * Check if the taxonomy is from core taxonomies.
     * @param $slug string - taxonomy slug
     * @return boolean
	 */
    public static function is_core_taxonomy($slug) {
        
        return $slug == self::$categories_tax || $slug == self::$taxonomies_list_tax || $slug == self::$ages_tax;       
    
    }
    
////////////////////////////
    /**
	 * Get posts min or max price from based on search request
     * 
     * @param array $args - search params
     * @return float
	 */
    public static function get_posts_range_price($args = array(), $flag = 'max') {
        
        $args['paged'] = 1;
        $args['posts_per_page'] = 1;
        $args['sort'] = 'price_from';
        $args['sort_by'] = $flag == 'max' ? 'DESC' : 'ASC';
        
        unset($args['min_price']);
        unset($args['max_price']);
        
        $posts = self::get_posts($args);
        
        return !empty($posts) ? $posts[0]['discount_price_from'] : 0;
        
    }           
    
////////////////////////////
    /**
	 * Get posts - main search query.
     * @param array $args_array
     * @return array
	 */
    public static function get_posts($args_array = array()) {
        global $wpdb;
        
        $output = array();
        
        $args = wp_parse_args( $args_array, array(
            'date_from' => '', //// d/m/Y or m/d/Y format
            'date_to' => '',
            'time_from' => '00:00',
            'time_to' => '00:00',
            'categories' => array(), //// term_taxonomy_ids from categories
            'terms' => array(), //// term_taxonomy_ids from custom taxonomies in $taxonomies_list
            'posts_per_page' => BABE_Settings::$settings['results_per_page'],
            'guests' => 1,
            'sort' => 'price_from', /// price_from, rating
            'sort_by' => 'ASC',
            'min_price' => '',
            'max_price' => '',
            'post__in' => array(),
        ));
        
        ////// sanitize args
        
        $guests = max(1, intval($args['guests']));
        
        if (!$args['date_from']){
            $guests = 0;
        }
        
        $date_now_obj = BABE_Functions::datetime_local();
        $date_to_obj_alt = clone $date_now_obj;
        $date_to_obj_alt->modify("+1 month");
        
        $date_from = BABE_Calendar_functions::isValidDate($args['date_from'], BABE_Settings::$settings['date_format']) ? BABE_Calendar_functions::date_to_sql($args['date_from']) : $date_now_obj->format("Y-m-d");
        
        $date_to = BABE_Calendar_functions::isValidDate($args['date_to'], BABE_Settings::$settings['date_format']) ? BABE_Calendar_functions::date_to_sql($args['date_to']) : $date_to_obj_alt->format("Y-m-d");
        
        if(BABE_Calendar_functions::isValidTime($args['time_from'], 'H:i')){
          $date_from .= ' '.$args['time_from'];
        }
        
        if(BABE_Calendar_functions::isValidTime($args['time_to'], 'H:i')){
          $date_to .= ' '.$args['time_to'];
        }
        
        $date_from_obj = new DateTime($date_from);
        $date_to_obj = new DateTime($date_to);
        
        //// add one year date range optimization
        
        $datetime_from = $date_from_obj->format('Y-m-d H:i:s');
        $datetime_to = $date_to_obj->format('Y-m-d H:i:s');
        
        $date_now = $date_now_obj->format('Y-m-d H:i:s');
        
        $main_age_id = self::get_main_age_id(); 
        $age_id_statement = $main_age_id ? "(age_id = 0 OR age_id = ".$main_age_id.")" : "(age_id = 0)";
        
        $args['paged'] = get_query_var('paged') ? absint(get_query_var('paged')) : 1;
        $posts_per_page = intval($args['posts_per_page']);
        $posts_per_page = $posts_per_page < 1 ? -1 : $posts_per_page;
        $offset = max(0, intval($args['paged'])-1)*$posts_per_page;
        
        $limit_clauses = $posts_per_page == -1 ? "" : "
        LIMIT ".$offset.", ".$posts_per_page."   
";
        
        $sort = $args['sort'] == 'price_from' ? 'discount_price_from' : ($args['sort'] == 'rating' ? 'pmr.rating' : 'discount_price_from');
        $sort_by = $args['sort_by'] == 'ASC' ? ' ASC' : ' DESC';
        $order_by = 'ORDER BY '.$sort.$sort_by;
        
        $terms_filter = '';
        
        if(!empty($args['terms'])){
            
            $args['terms'] = array_map( 'intval', $args['terms'] );
            $args['terms'] = array_unique($args['terms']);
            
         foreach($args['terms'] as $key => $term_id){   
        $terms_filter .= "        INNER JOIN
        (
        SELECT object_id AS tt".$term_id."_object_id, term_taxonomy_id AS tt".$term_id."_term_taxonomy_id
        FROM ".$wpdb->term_relationships."
        WHERE term_taxonomy_id = ".$term_id."
        ) tt".$term_id." ON posts.ID = tt".$term_id.".tt".$term_id."_object_id
        ";
        }
        
        }
        
        if(!empty($args['categories'])){
           $args['categories'] = array_map( 'intval', $args['categories'] );
           $categories_filter = " AND term_taxonomy_id IN (".implode(", ", $args['categories']).")";
        } else {
           $categories_filter = ''; 
        }
        
        if(!empty($args['post__in'])){
           $args['post__in'] = array_map( 'intval', $args['post__in'] );
           $post__in_filter = " AND posts.ID IN (".implode(", ", $args['post__in']).")";
        } else {
           $post__in_filter = ''; 
        }
        
        $min_price_filter = "";
        if($args['min_price']){
            $min_price_filter = "
            AND ((trm_min.price_from*(100-COALESCE(pmd.discount, 0))*(100+COALESCE(tm2.categories_add_taxes, 0)*COALESCE(tm3.categories_tax, 0)))/(100*100)) >= ".absint($args['min_price']);
        }
        
        $max_price_filter = "";
        if($args['max_price']){
            $max_price_filter = "
            AND ((trm_min.price_from*(100-COALESCE(pmd.discount, 0))*(100+COALESCE(tm2.categories_add_taxes, 0)*COALESCE(tm3.categories_tax, 0)))/(100*100)) <= ".absint($args['max_price']);
        }
        
        ///// create query
        
        $query = "SELECT *, (trm_min.price_from*(100-COALESCE(pmd.discount, 0))*(100+COALESCE(tm2.categories_add_taxes, 0)*COALESCE(tm3.categories_tax, 0)))/(100*100) AS discount_price_from 
        FROM ".$wpdb->posts." posts 
        
        INNER JOIN #every our post assigned to terms from categories and other taxonomies
        (
        SELECT object_id AS tr_object_id, term_taxonomy_id AS tr_term_taxonomy_id
        FROM ".$wpdb->term_relationships."
        ) tr ON posts.ID = tr.tr_object_id  
        
        INNER JOIN #we need only our post type with categories
        (
        SELECT term_taxonomy_id AS ct_term_taxonomy_id, term_id AS ct_term_id
        FROM ".$wpdb->term_taxonomy."
        WHERE taxonomy = '".self::$categories_tax."'".$categories_filter."
        ) ct ON ct.ct_term_taxonomy_id = tr.tr_term_taxonomy_id
        
        INNER JOIN #get category slug
        (
        SELECT slug AS category_slug, term_id AS ctt_term_id
        FROM ".$wpdb->terms."
        ) ctt ON ctt.ctt_term_id = ct.ct_term_id
        
        ".$terms_filter."
        
        INNER JOIN # get rates
        (
        SELECT *
        FROM ".BABE_Prices::$table_rate."
        WHERE 1=1 AND ((rate_type = 'b_seasonal' AND date_to >= '".$date_from."' AND date_from <= '".$date_to."') OR (rate_type = 'a_base')) #(rate_type = 'c_event' AND date_from >= '".$date_from."' AND date_to <= '".$date_to."') OR 
        ) t_rate ON posts.ID = t_rate.booking_obj_id
        
        INNER JOIN # get min price from
        (
        SELECT rate_id AS trm_rate_id, MIN(price_from) AS price_from
        FROM ".BABE_Prices::$table_rate_meta."
        WHERE 1=1 AND price_type != 'd_monthly' AND ".$age_id_statement."
        GROUP BY rate_id
        ORDER BY price_from ASC
        ) trm_min ON t_rate.rate_id = trm_min.trm_rate_id
        
        LEFT JOIN #add categories_booking_rule meta
        (
        SELECT meta_value AS categories_booking_rule, term_id AS tm_term_id
        FROM ".$wpdb->termmeta."
        WHERE meta_key = 'categories_booking_rule'
        ) tm ON ct.ct_term_id = tm.tm_term_id
        
        LEFT JOIN #add categories_add_taxes meta
        (
        SELECT CAST(meta_value AS UNSIGNED) AS categories_add_taxes, term_id AS tm2_term_id
        FROM ".$wpdb->termmeta."
        WHERE meta_key = 'categories_add_taxes'
        ) tm2 ON ct.ct_term_id = tm2.tm2_term_id
        
        LEFT JOIN #add categories_tax meta
        (
        SELECT CAST(meta_value AS UNSIGNED) AS categories_tax, term_id AS tm3_term_id
        FROM ".$wpdb->termmeta."
        WHERE meta_key = 'categories_tax'
        ) tm3 ON ct.ct_term_id = tm3.tm3_term_id
        
        LEFT JOIN #get rating
        (
        SELECT CAST(meta_value AS DECIMAL(3,2)) AS rating, post_id AS pmr_post_id 
        FROM ".$wpdb->postmeta."
        WHERE meta_key = '_rating'
        ) pmr ON posts.ID = pmr.pmr_post_id
        
        LEFT JOIN #get discount
        (
        SELECT discount, date_from AS discount_date_from, date_to AS discount_date_to, booking_obj_id AS discount_obj_id
        FROM ".BABE_Prices::$table_discount."
        WHERE date_from <= '".$date_now."' AND date_to >= '".$date_now."'
        ) pmd ON posts.ID = pmd.discount_obj_id
        
        LEFT JOIN # get rule
        (
        SELECT *
        FROM ".BABE_Booking_Rules::$table_booking_rules."
        ) rules ON rules.rule_id = tm.categories_booking_rule
        
        LEFT JOIN #get items number
        (
        SELECT GREATEST(CAST(COALESCE(meta_value, 1) AS DECIMAL), 1) AS items_number, post_id AS pmt_post_id, meta_key AS pmt_meta_key
        FROM ".$wpdb->postmeta."
        ) pmt ON posts.ID = pmt.pmt_post_id AND pmt.pmt_meta_key = CONCAT('items_number_', ctt.category_slug)
        
        INNER JOIN #get max guests
        (
        SELECT CAST(meta_value AS DECIMAL) AS guests, post_id AS pm_post_id 
        FROM ".$wpdb->postmeta."
        WHERE meta_key = 'guests'
        ) pm ON posts.ID = pm.pm_post_id AND pm.guests >= ".$guests."
        
        INNER JOIN # get av cal
        (
        SELECT booking_obj_id AS obj_id
        FROM ".BABE_Calendar_functions::$table_av_cal."
        WHERE in_schedule=1 AND date_from >= '".$date_from."' AND date_from <= '".$date_to."'
        GROUP BY booking_obj_id
        ) av_cal ON av_cal.obj_id = posts.ID
        
        LEFT JOIN # get av cal ex
        (
        SELECT booking_obj_id AS ex_obj_id, date_from AS av_date_from, MAX(guests) AS ex_guests_present
        FROM ".BABE_Calendar_functions::$table_av_cal."
        WHERE in_schedule=1
        GROUP BY booking_obj_id
        ) av_cal_ex ON av_cal_ex.ex_obj_id = posts.ID AND ( rules.booking_mode = 'object' AND av_cal_ex.ex_guests_present > (pm.guests*pmt.items_number - ".$guests." ) OR rules.booking_mode != 'object' AND av_cal_ex.ex_guests_present > (pm.guests - ".$guests." ) ) AND rules.basic_booking_period != 'recurrent_custom' AND rules.basic_booking_period != 'night' AND av_cal_ex.av_date_from >= DATE_SUB('".$datetime_from."', INTERVAL rules.hold HOUR) AND av_cal_ex.av_date_from <= DATE_ADD('".$datetime_to."', INTERVAL rules.hold HOUR)
        
        LEFT JOIN # get av cal ex2
        (
        SELECT booking_obj_id AS ex2_obj_id, MAX(guests) AS ex2_guests_present
        FROM ".BABE_Calendar_functions::$table_av_cal."
        WHERE in_schedule=1 AND date_from >= '".$date_from."' AND date_from < '".$date_to."'
        GROUP BY booking_obj_id
        ) av_cal_ex2 ON av_cal_ex2.ex2_obj_id = posts.ID AND ( rules.booking_mode = 'object' AND av_cal_ex2.ex2_guests_present > (pm.guests*pmt.items_number - ".$guests." ) OR rules.booking_mode != 'object' AND av_cal_ex2.ex2_guests_present > (pm.guests - ".$guests." ) ) AND rules.basic_booking_period = 'night'
        
        LEFT JOIN # get av cal ex3 for recurrent_custom
        (
        SELECT booking_obj_id AS ex3_obj_id, MIN(guests) AS ex3_guests_present
        FROM ".BABE_Calendar_functions::$table_av_cal."
        WHERE in_schedule=1 AND date_from >= '".$datetime_from."' AND date_from <= '".$datetime_to."'
        GROUP BY booking_obj_id
        ) av_cal_ex3 ON av_cal_ex3.ex3_obj_id = posts.ID AND av_cal_ex3.ex3_guests_present > (pm.guests - ".$guests." ) AND rules.basic_booking_period = 'recurrent_custom'
        
        WHERE (
          (posts.post_status = 'publish'
          AND
           posts.post_type = '".self::$booking_obj_post_type."')".$post__in_filter.$min_price_filter.$max_price_filter."
           AND av_cal_ex.ex_obj_id IS NULL
           AND av_cal_ex2.ex2_obj_id IS NULL
           AND av_cal_ex3.ex3_obj_id IS NULL
        )
        
        GROUP BY posts.ID
        ";
/*
*/
        // run main query
        $output = $wpdb->get_results($query.$order_by.$limit_clauses, ARRAY_A);
        
        // run count query
        self::$get_posts_count = $wpdb->get_var("SELECT COUNT(discount_price_from) AS total_count FROM (".$query.") AS a");
        
        self::$get_posts_pages = $posts_per_page == -1 ? 1 : ceil(self::$get_posts_count/$posts_per_page);
        
        return $output;
    } 
    
/////is_post_booking_obj////    
    /**
	 * Is the post type a booking obj post type?
     * @param int $post_id
     * @return boolean
	 */
    public static function is_post_booking_obj($post_id){
        
        $post_id = absint($post_id); 
        
        return self::$booking_obj_post_type == get_post_type($post_id) ? true : false;
    }
    
/////is_post_order////    
    /**
	 * Is the post type an order post type?
     * @param int $post_id
     * @return boolean
	 */
    public static function is_post_order($post_id){
        
        $post_id = absint($post_id); 
        
        return self::$order_post_type == get_post_type($post_id) ? true : false;
    }    
    
/////////////////////    
    /**
	 * Get post services.
     * @param array $post
     * @return array
	 */
    public static function get_post_services($post) {
        global $wpdb;
        
        $output = array();
          
          if (!empty($post) && isset($post['ID']) && isset($post['services']) && !empty($post['services'])){
            
            $ids = (array)$post['services'];
            
            if (!empty($ids)){
                $ids = array_map( 'intval', $ids );
        ////
        $query = "SELECT * 
        FROM ".$wpdb->posts." posts
        
        LEFT JOIN #get service_type
        (
        SELECT meta_value AS service_type, post_id AS pms_post_id 
        FROM ".$wpdb->postmeta."
        WHERE meta_key = 'service_type'
        ) pms ON posts.ID = pms.pms_post_id
        
        LEFT JOIN #get price_type
        (
        SELECT meta_value AS price_type, post_id AS pm_post_id 
        FROM ".$wpdb->postmeta."
        WHERE meta_key = 'price_type'
        ) pm ON posts.ID = pm.pm_post_id
        
        LEFT JOIN #get prices
        (
        SELECT meta_value AS prices, post_id AS pmp_post_id 
        FROM ".$wpdb->postmeta."
        WHERE meta_key = 'prices'
        ) pmp ON posts.ID = pmp.pmp_post_id
        
        WHERE (
          (posts.post_status = 'publish'
          AND
           posts.post_type = '".self::$service_post_type."')
          AND posts.ID IN (".implode(", ", $ids).")
        )
        
        GROUP BY posts.ID
        ORDER BY service_type ASC  
";
        /////
     
        $results = $wpdb->get_results($query, ARRAY_A);
        foreach($results as $i => $result){
            $output[$i] = $result;
            $output[$i]['prices'] = maybe_unserialize($result['prices']);
            if (!isset($output[$i]['prices'][0])){
                $output[$i]['prices'][0] = 0;
            }
            unset($output[$i]['pmp_post_id']);
            unset($output[$i]['pm_post_id']);
            unset($output[$i]['pms_post_id']);
        }
        
            }  //// end if !empty($ids)
            
          } //// end if !empty($post) 
        
        return $output;
    }
    
//////////////////////////////
    /**
	 * Get post FAQs.
     * @param array $post
     * @return array
	 */
    public static function get_post_faq($post) {
        
        $output = array();
          
        if (!empty($post) && isset($post['ID']) && isset($post['faq']) && !empty($post['faq'])){
            
          $ids = (array)$post['faq'];
            
          if (!empty($ids)){
             $ids = array_map( 'intval', $ids );
                
             $args = array(
                'post_type'   => self::$faq_post_type,
                'numberposts' => -1,
                'post__in' => $ids,
                'post_status' => 'publish',
                'orderby' => 'menu_order',
                'order' => 'ASC',
             );
             
             $args = apply_filters('babe_get_post_faq_args', $args, $post);
             
             $posts_obj = get_posts( $args );
             $output = json_decode(json_encode($posts_obj), true);
          } //// end if !empty $ids   
       } /// end if !empty $post      
             
       return $output;
    
    }
    
//////////////////////////////
    /**
	 * Get post related items.
     * 
     * @param array $post
     * @return array
	 */
    public static function get_post_related($post) {
        
        $output = array();
          
        if (!empty($post) && isset($post['ID']) && isset($post['related_items']) && !empty($post['related_items'])){
            
          $ids = (array)$post['related_items'];
            
          if (!empty($ids)){
             $ids = array_map( 'intval', $ids );
                
             $args = array(
                'post__in' => $ids,
             );
             
             $args = apply_filters('babe_get_post_related', $args, $post);
             
             $output = self::get_posts( $args );
          } //// end if !empty $ids   
       } /// end if !empty $post      
             
       return $output;
    
    }             
    
/////////////////////    
    /**
	 * Get post meeting points.
     * @param array $post
     * @return array
	 */
    public static function get_post_meeting_points($post) {
        $output = array();
          
          if (!empty($post) && isset($post['meeting_points']) && isset($post['meeting_place']) && $post['meeting_place'] == 'point'){
           
           foreach($post['meeting_points'] as $meeting_point){
            
              $place_id = $meeting_point['place'];
              $address_arr = get_post_meta($place_id, 'address', 1);
              $description = get_post_meta($place_id, 'description', 1);
              $lat = isset($address_arr['latitude']) ? $address_arr['latitude'] : 0;
              $lng = isset($address_arr['longitude']) ? $address_arr['longitude'] : 0;
              $address = isset($address_arr['address']) ? apply_filters('translate_text', $address_arr['address']) : '';
              
              $times = array();
              if (!empty($post['schedule'])){
                 foreach($post['schedule'] as $schedule){
                    foreach($schedule as $time){
                      $date_tmp = new DateTime('2017-01-01 '.$time);
                      $modify_string = '';
                      $modify_string .= isset($meeting_point['time_shift']['h']) ? '-'.intval($meeting_point['time_shift']['h']).' hour ' : '';
                      $modify_string .= isset($meeting_point['time_shift']['i']) ? '-'.intval($meeting_point['time_shift']['i']).' minute' : '';
                      if ($modify_string){
                          $date_tmp->modify($modify_string);
                      }
                      $times[$time] = $date_tmp->format(get_option('time_format'));
                    }
                 }
              }
              $times = array_unique($times);
              
              $output[$place_id] = array(
                 'title' => get_the_title($place_id),
                 'address' => $address,
                 'lat' => $lat,
                 'lng' => $lng,
                 'description' => $description,
                 'times' => $times,
                 'permalink' => get_permalink($place_id),
                 'point_id' => $place_id,
              );
           } //// end foreach $post['meeting_points']
            
          } //// end if !empty($post) 
        
        return $output;
    }
    
////////////////////////////
    /**
	 * Get post available times.
     * @param array $post
     * @return array
	 */
    public static function get_post_av_times($post) {
        $output = array();
        
        if (!empty($post) && isset($post['schedule']) && !empty($post['schedule'])){            
            foreach($post['schedule'] as $day_num => $time_arr){
                foreach($time_arr as $time){
                    $output[] = $time;
                }
            }
            
            $output = array_unique($output);
            sort($output);
        }
        
        return $output;
    }        
     
////////////////////////////
    /**
	 * Get post available days.
     * @param array $post
     * @return array
	 */
    public static function get_post_av_days($post) {
        $output = array();
        
        //$post = self::get_post($post_id);
        if (!empty($post) && isset($post['schedule']) && !empty($post['schedule'])){            
            $week_days = BABE_Calendar_functions::get_week_days_arr();
            foreach($post['schedule'] as $day_num => $time_arr){
                $output[$day_num] = $week_days[$day_num];
            }
        }
        
        return $output;
    }
    
////////////////////////////
    /**
	 * Get post available days.
     * @param array $post
     * @return string
	 */
    public static function get_post_av_days_str($post) {
        $week_days = self::get_post_av_days($post);
        $week_days_string = implode(', ', $week_days);  
        return sizeof($week_days) == 7 ? __('Daily', BABE_TEXTDOMAIN) : $week_days_string;
    }
    
////////////////////////////
    /**
	 * Get post duration.
     * @param array $post
     * @return array
	 */
    public static function get_post_duration($post) {
        $output = '';
        
        $arr = array();
        
        if (!empty($post) && isset($post['duration']) && !empty($post['duration'])){            
            if ($post['duration']['d']) {
                $arr[] = $post['duration']['d'].' '.__('days', BABE_TEXTDOMAIN);
            }
            if ($post['duration']['h']) {
                $arr[] = $post['duration']['h'].' '.__('hours', BABE_TEXTDOMAIN);
            }
            if ($post['duration']['i']) {
                $arr[] = $post['duration']['i'].' '.__('minutes', BABE_TEXTDOMAIN);
            }
            $output .= implode(' ', $arr);
        }
        
        return $output;
    }
    
////////////////////////////
    /**
	 * Get post items number
     * 
     * @param int|array post ID or post array
     * @return int
	 */
    public static function get_post_items_number($post) {
        $output = 1;
        
        if (is_numeric($post)){
            $post = self::get_post($post);
        }
        
       if ( !empty($post) && is_array($post) ){
        
            $rules_cat = BABE_Booking_Rules::get_rule_by_obj_id($post['ID']);
            
            $output = $rules_cat['rules']['booking_mode'] == 'object' && isset($post['items_number']) && absint($post['items_number']) ? absint($post['items_number']) : 1;
            
        }
        
        return $output;
    }                     
        
////////////////////////////
    /**
	 * Get post with all meta.
     * @param int $post_id
     * @return array
	 */
    public static function get_post($post_id) {
        global $wpdb;
        
        $output = array();
        
        $post_id = absint($post_id);
        
        if (isset(self::$post[$post_id])){
            return self::$post[$post_id];  
        }
        
        if ($post_id > 0){
        
        $query = "SELECT * 
        FROM ".$wpdb->posts." posts
        WHERE (
          (posts.ID = ".$post_id."
          AND
           posts.post_status = 'publish')
        )";
        
        $post = $wpdb->get_results($query, ARRAY_A);
        
        if (!empty($post)){
            
          $output = $post[0];
          $meta = array();
          
          $term = self::get_post_category($post_id);
          
          if (!empty($term)){
            
          $slug = '_'.$term->slug; 
        
          $query = "SELECT * 
          FROM ".$wpdb->postmeta." pm
          WHERE pm.post_id = ".$post_id."
          ORDER BY pm.meta_key ASC";
        
          $result = $wpdb->get_results($query, ARRAY_A);
          foreach($result as $row){
             if(substr($row['meta_key'], 0, 1) != '_' && substr($row['meta_key'], 0, 4) != 'tmp_'){
                $meta_key = str_replace($slug, '', $row['meta_key']);                
                $meta[$meta_key] = maybe_unserialize($row['meta_value']);
             }
          }
          $output = $output + $meta;
          
          } ////  end if !empty($term)
          
        } ///// end if !empty($post)
        
        }
        
        self::$post[$post_id] = $output;
        
        return $output;
    }
    
////////////////////////////
    /**
	 * Get all post meta.
     * @param int $post_id
     * @return array
	 */
    public static function get_post_meta($post_id) {
        global $wpdb;
        
        $output = array();
        
        $post_id = absint($post_id);
        
        if (isset(self::$post_meta[$post_id])){
            return self::$post_meta[$post_id];  
        }
        
        if ($post_id > 0){

          $meta = array();
          
          $term = self::get_post_category($post_id);
          
          if (!empty($term)){
            
          $slug = '_'.$term->slug; 
        
          $query = "SELECT * 
          FROM ".$wpdb->postmeta." pm
          WHERE pm.post_id = ".$post_id."
          ORDER BY pm.meta_key ASC";
        
          $result = $wpdb->get_results($query, ARRAY_A);
          foreach($result as $row){
             if(substr($row['meta_key'], 0, 1) != '_' && substr($row['meta_key'], 0, 4) != 'tmp_'){
                $meta_key = str_replace($slug, '', $row['meta_key']);                
                $meta[$meta_key] = maybe_unserialize($row['meta_value']);
             }
          }
          $output = $meta;
          
          } ////  end if !empty($term)
        
        }
        
        self::$post_meta[$post_id] = $output;
        
        return $output;
    }
    
////////////////////////////
    /**
	 * Get post discount.
     * @param int $post_id
     * @return array
	 */
    public static function get_post_discount($post_id) {
        global $wpdb;
        
        if (isset(self::$post_discount[$post_id])){
            return self::$post_discount[$post_id];  
        }
        
        $date_now_obj = BABE_Functions::datetime_local();
        $date_now = $date_now_obj->format('Y-m-d H:i:s');
        
        /// get discount
        $discount_db = $wpdb->get_row("SELECT discount, date_from AS discount_date_from, date_to AS discount_date_to, booking_obj_id AS discount_obj_id FROM ".BABE_Prices::$table_discount." WHERE booking_obj_id = '".$post_id."' AND date_from <= '".$date_now."' AND date_to >= '".$date_now."'", ARRAY_A); 
        
        $output = !empty($discount_db) ? $discount_db : array('discount' => 0, 'discount_obj_id' => 0);
        unset($output['discount_obj_id']);
        
        self::$post_discount[$post_id] = $output;

        return $output;
    }        
    
////////////////////////////
    /**
	 * Get post price from for nearest month.
     * @param int $post_id
     * @return array
	 */
    public static function get_post_price_from($post_id) {
        global $wpdb;
        
        $rules_cat = BABE_Booking_Rules::get_rule_by_obj_id($post_id); 
        $tax_am = isset($rules_cat['category_meta']['categories_add_taxes']) && $rules_cat['category_meta']['categories_add_taxes'] && isset($rules_cat['category_meta']['categories_tax']) ? floatval($rules_cat['category_meta']['categories_tax'])/100 : 0;
        
        $main_age_id = self::get_main_age_id(); 
        $age_id_statement = $main_age_id ? "(age_id = 0 OR age_id = ".$main_age_id.")" : "(age_id = 0)";
        
        $date_now_obj = BABE_Functions::datetime_local();
        
        $date_now = $date_now_obj->format('Y-m-d H:i:s');
        
        $date_to_obj = new DateTime($date_now_obj->format('Y-m-d'));
        $date_to_obj->modify("+1 month");
        
        $datetime_from = $date_now_obj->format('Y-m-d 00:00:00');
        $datetime_to = $date_to_obj->format('Y-m-d H:i:s');
        
        $query = "SELECT *, (trm_min.price_from*(100-CAST(COALESCE(pmd.discount, 0) AS UNSIGNED))/100) AS discount_price_from 
        FROM ".$wpdb->posts." posts
        
        LEFT JOIN #get discount
        (
        SELECT discount, date_from AS discount_date_from, date_to AS discount_date_to, booking_obj_id AS discount_obj_id
        FROM ".BABE_Prices::$table_discount."
        WHERE date_from <= '".$date_now."' AND date_to >= '".$date_now."'
        ) pmd ON posts.ID = pmd.discount_obj_id
            
        INNER JOIN # get rates
        (
        SELECT *
        FROM ".BABE_Prices::$table_rate."
        WHERE 1=1 AND ((rate_type = 'b_seasonal' AND date_to >= '".$datetime_from."' AND date_from <= '".$datetime_to."') OR (rate_type = 'a_base')) #(rate_type = 'c_event' AND date_from >= '".$datetime_from."' AND date_to <= '".$datetime_to."') OR 
        ) t_rate ON posts.ID = t_rate.booking_obj_id
        INNER JOIN # get min price from
        (
        SELECT rate_id, MIN(price_from) AS price_from
        FROM ".BABE_Prices::$table_rate_meta."
        WHERE 1=1 AND price_type != 'd_monthly' AND ".$age_id_statement."
        GROUP BY rate_id
        ORDER BY price_from ASC
        ) trm_min ON t_rate.rate_id = trm_min.rate_id
        
        WHERE (
          (posts.ID = ".$post_id."
          AND
           posts.post_status = 'publish')
        )
        
        GROUP BY posts.ID
        ORDER BY discount_price_from ASC
        LIMIT 0, 1   
";     
        $results = $wpdb->get_results($query, ARRAY_A);
        
        return !empty($results) ? array(
           'price_from' => ($results[0]['price_from'] + round($results[0]['price_from'] * $tax_am, 2)),
           'discount' => $results[0]['discount'],
           'discount_price_from' => ($results[0]['discount_price_from'] + round($results[0]['discount_price_from'] * $tax_am, 2)),
           'discount_date_from' => $results[0]['discount_date_from'],
           'discount_date_to' => $results[0]['discount_date_to'],
        ) : array();
    }    
    
////////////////////////////
    /**
	 * Get post category.
     * @param int $post_id
     * @return mixed (Object or string)
	 */
    public static function get_post_category($post_id) {

        $categories_arr = get_the_terms($post_id, self::$categories_tax);
        if (!empty($categories_arr) && !is_wp_error($categories_arr)){
           return $categories_arr[0];   
        }  
        
        return '';
    }        

////////////////////////////
    /**
	 * Get post terms from all custom taxonomies or one taxonomy
     * @param int $post_id
     * @param string $taxonomy_slug
     * @return array
	 */
    public static function get_post_terms($post_id, $taxonomy_slug = '') {
        
        $output = array();
        
        $taxonomies_list = self::$taxonomies_list;
        
        $rules = BABE_Booking_Rules::get_rule_by_obj_id($post_id);
        
        if (!empty($rules) && isset($rules['category_meta']['categories_taxonomies'])){
        foreach($rules['category_meta']['categories_taxonomies'] as $taxonomy_id){
            $taxonomy = $taxonomies_list[$taxonomy_id]; 
            // slug, select_mode, frontend_style, frontend_class, gmap_active
            if (!$taxonomy_slug || ($taxonomy_slug && $taxonomy_slug == $taxonomy['slug'])){
            
            $terms = get_the_terms($post_id, $taxonomy['slug']);
            if (!empty($terms) && !is_wp_error($terms)){
               $output[$taxonomy['slug']] = $taxonomy;
               foreach($terms as $term){
                   $output[$taxonomy['slug']]['terms'][$term->term_id] = array(
                    'term_id' => $term->term_id,
                    'term_taxonomy_id' => $term->term_taxonomy_id,
                    'name' => apply_filters('translate_text', $term->name),
                    'slug' => $term->slug,
                    'description' => apply_filters('translate_text', $term->description),
                    'parent' => $term->parent,
                    'count' => $term->count,
                   );
                   $term_vals = get_term_meta($term->term_id);
                   foreach($term_vals as $key=>$val){
                    $output[$taxonomy['slug']]['terms'][$term->term_id][$key] = $val[0];
                   }
                   $output[$taxonomy['slug']]['terms'][$term->term_id]['image_id'] = get_term_meta($term->term_id, 'image_id', 1);
               } 
            } else {
                $output[$taxonomy['slug']] = array();
            }
            
            } //// end if $taxonomy_slug
        }
        }     
        
        return !empty($output) && $taxonomy_slug ? $output[$taxonomy_slug] : $output;
    }
    
////////////////////////////
    /**
	 * Get terms hierarchy with active last level children
     * @param array $args
     * @param array $value - selected term ids
     * @return string
	 */
    public static function get_terms_children_hierarchy($args, $value = array()){
    
    $output = '';
    
    // make sure we specify each part of the value we need.
	  $args = wp_parse_args( $args, array(
		'taxonomy' => '',
		'parent_term_id' => 0,
		'level' => 0,
        'view' => '', // 'select', 'multicheck' or 'list'
        'add_count' => '',
        'option_all' => '',
        'option_all_value' => 0,
        'option_all_title' => '',
        'id' => '',
        'class' => '',
        'class_item' => 'term_item',
        'class_item_selected' => 'term_item_selected',
        'name' => '',
        'prefix_char' => ' ',
        'term_id_name' => 'term_id',
        'data-conditional-id' => '',
        'data-conditional-value' => '',
	  ) );
      
      if ($args['taxonomy']){
      
      $args['view'] = 'select' || 'multicheck' || 'list' ? $args['view'] : 'list';    
      $args['name'] = !$args['name'] ? $args['taxonomy'].'_terms' : $args['name'];
      $args['id'] = $args['id'] ? $args['id'] : $args['name'];
      $args['term_id_name'] = $args['term_id_name'] == 'term_id' ? $args['term_id_name'] : 'term_taxonomy_id';
      
      $conditional_attr = $args['data-conditional-id'] && $args['data-conditional-value'] ? ' data-conditional-id="'.$args['data-conditional-id'].'" data-conditional-value="'.$args['data-conditional-value'].'"' : ''; 
    
    $terms_all = get_terms( array(
          'taxonomy' => $args['taxonomy'],
          'hide_empty' => false,
          'parent' => $args['parent_term_id']
      ) );
      
    $prefix = substr($args['prefix_char'].$args['prefix_char'].$args['prefix_char'].$args['prefix_char'].$args['prefix_char'].$args['prefix_char'].$args['prefix_char'], 0, $args['level']*mb_strlen($args['prefix_char']));    
      
    foreach ( $terms_all as $term ) {
        $add_term_count = $args['add_count'] ? ' (' . absint( $term->count ) . ')' : '';
        
        $new_args = $args;
        $new_args['level'] = $args['level'] + 1;
        $new_args['parent_term_id'] = $term->term_id;
        
        $in_array = in_array($term->{$args['term_id_name']}, (array)$value);
        $class = $in_array ? $args['class_item'].' '.$args['class_item_selected'] : $args['class_item'];
        $class .= ' '.$args['class_item'].'_level_'.$args['level'];
        
        $children = self::get_terms_children_hierarchy($new_args, $value);
        
        $term_name = esc_attr( apply_filters('translate_text', $term->name) );
        
        $term_name = apply_filters('babe_get_terms_hierarchy_term_name', $term_name, $term->term_id, $args);
        
        if($args['view'] == 'select'){
            
            $selected = $in_array ? ' selected' : '';
            $disabled = $children ? ' disabled' : '';
            $class = $children ? $class.' term_item_disabled' : $class;
            
            $output .= '<option value="'.$term->{$args['term_id_name']}.'" class="'.$class.'"'.$selected.$disabled.'>'.$prefix.$term_name.$add_term_count.'</option>';
            $output .= $children;
            
        } elseif ($args['view'] == 'multicheck'){ 
            
            $checked = $in_array ? ' checked="checked"' : '';
            $class = !$children ? $class.' term_item_checkbox' : $class;
            
            $output .= '<div class="'.$class.'" data-id="'.$term->term_id.'" data-parent-id="'.$term->parent.'">'.$prefix;
            $output .= !$children ? '<input type="checkbox" id="'.$args['name'].'_'.$term->{$args['term_id_name']}.'" name="'.$args['name'].'[]" value="'.$term->{$args['term_id_name']}.'"'.$checked.$conditional_attr.'><label for="'.$args['name'].'_'.$term->{$args['term_id_name']}.'">' : '';
            $output .= $term_name.$add_term_count;
            $output .= !$children ? '</label></div>' : '</div>';
            
            $output .= $children;
        } else {
            
            $output .= '<li class="'.$class.'" data-id="'.$term->{$args['term_id_name']}.'">'.$prefix.$term_name.$add_term_count.$children.'</li>';
            
        }  
	} /// end foreach $terms_all
    
    if ($output && $args['view'] == 'list') {
        $add_id = $args['level'] ? '_'.$args['parent_term_id'] : '';
        
        if ($args['level'] === 0){
        
        $pre_option = $args['option_all'] && $args['option_all_title'] ? '<li class="'.$args['class_item'].' term_item_all" data-id="'.$args['option_all_value'].'">'.$args['option_all_title'].'</li>' : '';
                
        $output = '<ul id="'.$args['id'].$add_id.'" class="'.$args['class'].'" data-parent="'.$args['parent_term_id'].'">
        '.$pre_option.$output.'
        </ul>';
        
        } else {
           $output = '
           <ul id="'.$args['id'].$add_id.'" class="'.$args['class'].'_'.$args['level'].'" >
             '.$output.'
           </ul>
           '; 
        }
        
    } elseif ($output && $args['level'] === 0){
        if($args['view'] == 'select'){
            
            $pre_option = $args['option_all'] && $args['option_all_title'] ? '<option value="'.$args['option_all_value'].'" class="'.$args['class_item'].'">'.$args['option_all_title'].'</option>' : '';
            
            $output = '<select id="'.$args['id'].'" name="'.$args['name'].'" class="'.$args['class'].'"'.$conditional_attr.'>
            '.$pre_option.$output.'
            </select>';
        
        } else {
            $output = '<div id="'.$args['id'].'" class="'.$args['class'].'">
            '.$output.'
            </div>';
        }
    }
    
    } /// end if $args['taxonomy']
    
    return $output;
    
    }
    
///////////////////////////
    /**
	 * Get excerpt from post content
     * @return string
	 */
    public static function get_post_excerpt($post, $length = 25){

    $the_excerpt = $post['post_content']; //Gets post_content to be used as a basis for the excerpt
    $excerpt_length = $length; //Sets excerpt length by word count
    $the_excerpt = strip_tags(strip_shortcodes($the_excerpt), '<p><ul><li><strong><a>'); //Strips tags and images
    $words = explode(' ', $the_excerpt, $excerpt_length + 1);

    if(count($words) > $excerpt_length) :
        array_pop($words);
       // array_push($words, '…');
        $the_excerpt = implode(' ', $words).'...';
    endif;

    $the_excerpt = BABE_Functions::closetags($the_excerpt);

    return $the_excerpt;
    
    }        
            
////////////////////////////
    
}

BABE_Post_types::init();
