<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * BABE_Prices Class.
 * 
 * @class 		BABE_Prices
 * @version		1.2.7
 * @author 		Booking Algorithms
 */

class BABE_Prices {
    
    // DB tables
    static $table_rate;
    
    static $table_rate_meta;
    
    static $table_discount;
    
    private static $nonce_title = 'prices-tpl-nonce';
    
//////////////////////////////
    /**
	 * Hook in tabs.
	 */
    public static function init() {  
       global $wpdb;
       self::$table_rate = $wpdb->prefix.'babe_rates';
       self::$table_rate_meta = $wpdb->prefix.'babe_rates_meta';
       self::$table_discount = $wpdb->prefix.'babe_discount';
       
       add_action( 'admin_enqueue_scripts', array( __CLASS__, 'enqueued_assets' ) );
       add_action( 'wp_enqueue_scripts', array( __CLASS__, 'wp_enqueue_scripts' ) );
       
       add_action( 'wp_ajax_get_price_details_form', array( __CLASS__, 'ajax_get_price_details_form'));
       add_action( 'wp_ajax_get_price_details_block', array( __CLASS__, 'ajax_get_price_details_block'));
       add_action( 'wp_ajax_save_rate', array( __CLASS__, 'ajax_save_rate'));
       add_action( 'wp_ajax_delete_rate', array( __CLASS__, 'ajax_delete_rate'));
       add_action( 'wp_ajax_check_base_rate', array( __CLASS__, 'ajax_check_base_rate'));
       
	}
    
///////////////////////////////////////
    /**
	 * Enqueue assets.
	 */
    public static function wp_enqueue_scripts() {
        
      if (isset($_GET['inner_page']) && $_GET['inner_page'] == 'edit-post' && isset($_GET['edit_post_id']) && $_GET['edit_post_id'] && BABE_Users::current_user_can_edit_post($_GET['edit_post_id'])){
        
     wp_enqueue_script( 'babe-modal-js', plugins_url( "js/babe-modal.js", BABE_PLUGIN ), array('jquery'), '1.0', true );
     
     wp_enqueue_style( 'babe-modal-style', plugins_url( "css/babe-modal.css", BABE_PLUGIN ));

     wp_enqueue_script( 'babe-prices-js', plugins_url( "js/admin/babe-admin-prices.js", BABE_PLUGIN ), array('jquery'), '1.0', true );
     wp_localize_script( 'babe-prices-js', 'babe_prices_lst', array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'date_format' => BABE_Settings::$settings['date_format'] == 'd/m/Y' ? 'dd/mm/yy' : 'mm/dd/yy',
            'nonce' => wp_create_nonce(self::$nonce_title)
         )
     );
     }

     }     

///////////////////////////////////////
    /**
	 * Enqueue assets admin.
	 */
    public static function enqueued_assets() {
        
     global $current_screen;  
        
     if (!empty($current_screen) && $current_screen->post_type == BABE_Post_types::$booking_obj_post_type && ($current_screen->base == 'post' || $current_screen->base == 'post-new')){
        
     wp_enqueue_script( 'babe-admin-modal-js', plugins_url( "js/babe-modal.js", BABE_PLUGIN ), array('jquery'), '1.0', true );
     
     wp_enqueue_style( 'babe-admin-modal-style', plugins_url( "css/babe-modal.css", BABE_PLUGIN ));

     wp_enqueue_script( 'babe-admin-prices-js', plugins_url( "js/admin/babe-admin-prices.js", BABE_PLUGIN ), array('jquery'), '1.0', true );
     wp_localize_script( 'babe-admin-prices-js', 'babe_prices_lst', array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'date_format' => BABE_Settings::$settings['date_format'] == 'd/m/Y' ? 'dd/mm/yy' : 'mm/dd/yy',
            'nonce' => wp_create_nonce(self::$nonce_title)
         )
     );
  
     wp_enqueue_style( 'babe-admin-prices-style', plugins_url( "css/admin/babe-admin-prices.css", BABE_PLUGIN ));
     }

     }    

//////////////////////////////
    /**
	 * Get rates for $booking_obj_id.
     * @param int $booking_obj_id.
     * @param datetime $datetime_from - MySQL datetime YYYY-MM-DD HH:MM:SS.
     * @param datetime $datetime_to - MySQL datetime YYYY-MM-DD HH:MM:SS.
     * @return array
	 */
    public static function get_rates($booking_obj_id, $datetime_from = '', $datetime_to = '') {
       global $wpdb;
       
       $clauses = (!$datetime_from || !$datetime_to) ? "" : " AND ((t_rate.rate_type = 'c_event' AND t_rate.date_from >= '".$datetime_from."' AND t_rate.date_to <= '".$datetime_to."') OR (t_rate.rate_type = 'b_seasonal' AND t_rate.date_to >= '".$datetime_from."' AND t_rate.date_from <= '".$datetime_to."') OR (t_rate.rate_type = 'a_base'))";
             
       $rates = $wpdb->get_results("SELECT * FROM ".self::$table_rate." t_rate INNER JOIN ".self::$table_rate_meta." t_rate_meta ON t_rate.rate_id = t_rate_meta.rate_id WHERE t_rate.booking_obj_id = '".$booking_obj_id."'".$clauses." ORDER BY t_rate.rate_type DESC, t_rate.date_from DESC, t_rate_meta.price_type DESC", ARRAY_A);
       
       $rates_arr = self::parse_db_rates_to_rates_arr($rates);
       
       return $rates_arr;   
	}
    
//////////////////////////////////////        
    /**
	 * Parse rates from DB to rates array
     * @param array $rates.
     * @return array
	 */
    public static function parse_db_rates_to_rates_arr($rates){
        
       $rates_arr = array();
       $before_rate_id = 0;
       $before_rate_type = '';
       $before_price_type = '';
       $i=0;
       
       foreach ($rates as $rate){
        if ($rate['rate_type'] != 'c_event'){ //// c_event is in the developement 
         if ($before_price_type != $rate['price_type']){            
            $before_price_type = $rate['price_type'];
            $price_type = substr($before_price_type, 2);
         }
        
         if ($before_rate_id != $rate['rate_id']){
            $i++;
            $rates_arr[$i]['rate_id'] = $rate['rate_id'];
            $rates_arr[$i]['rate_type'] = substr($rate['rate_type'], 2);
            $rates_arr[$i]['rate_title'] = $rate['rate_title'];
            $rates_arr[$i]['color'] = $rate['color'];
            $rates_arr[$i]['date_from'] = $rate['date_from'];
            $rates_arr[$i]['date_to'] = $rate['date_to'];
            $rates_arr[$i]['weekend_days'] = maybe_unserialize($rate['weekend_days']);
            $before_rate_id = $rate['rate_id'];
            if (empty($rates_arr[$i]['weekend_days'])) {
                $rates_arr[$i]['weekend_days'] = array();
            }
         }
         
         $rates_arr[$i][$price_type][$rate['age_id']] = $rate['price'];
         $rates_arr[$i][$price_type.'_from'][$rate['age_id']] = $rate['price_from'];
        } //// c_event is in the developement
       } /// end foreach
       
       return $rates_arr;
        
    }     
    
//////////////////////////////////////        
    /**
	 * Is there a base rate?
     * @param int $booking_obj_id.
     * @return boolean
	 */
    public static function base_rate_exist($booking_obj_id) {
        global $wpdb;
        
        $rate_id = $wpdb->get_var("SELECT t_rate.rate_id FROM ".self::$table_rate." t_rate INNER JOIN ".self::$table_rate_meta." t_rate_meta ON t_rate.rate_id = t_rate_meta.rate_id WHERE t_rate.booking_obj_id = '".$booking_obj_id."' AND t_rate.rate_type = 'a_base' AND t_rate_meta.price_type = 'a_base'");
        
        return $rate_id ? true : false;
    }

//////////////////////////////////////        
    /**
	 * Get base rate by $booking_obj_id
     * @param int $booking_obj_id.
     * @return array
	 */
    public static function get_base_rate($booking_obj_id) {
        global $wpdb;
        
        $rates = $wpdb->get_results("SELECT * FROM ".self::$table_rate." t_rate INNER JOIN ".self::$table_rate_meta." t_rate_meta ON t_rate.rate_id = t_rate_meta.rate_id WHERE t_rate.booking_obj_id = '".$booking_obj_id."' AND t_rate.rate_type = 'a_base' AND t_rate_meta.price_type = 'a_base'", ARRAY_A);
        
        $rates_arr = self::parse_db_rates_to_rates_arr($rates);
        
        return !empty($rates_arr) ? $rates_arr[1] : array();
        
    }    
    
///////////////////////////////////////    
    /**
	 * Create fields for price adding.
	 */
    public static function ajax_get_price_details_form(){
        
        $output = '';
        
        if (isset($_POST['post_id']) && isset($_POST['cat_slug']) && isset($_POST['nonce']) && wp_verify_nonce( $_POST['nonce'], self::$nonce_title ) && BABE_Users::current_user_can_edit_post($_POST['post_id'])){
             $post_id = intval($_POST['post_id']);
             $cat_slug = sanitize_text_field($_POST['cat_slug']);
             $rules = BABE_Booking_Rules::get_rule_by_cat_slug($cat_slug);
             
             if (!empty($rules)){
             // Array ( [rule_title] => Accomodation rule [basic_booking_period] => night [min_booking_period] => 1 [max_booking_period] => 0 [hold] => 0 [ages] => 1 [payment_model] => deposit_full [recurrent_payments] => 0 [booking_mode] => object [booking_rule_id] => 0 )
                //$output .= print_r($rules, 1);
                $basic_booking_period_title = '';
                
                if ($rules['basic_booking_period'] == 'night'){
                    $basic_booking_period_title = __('nights', BABE_TEXTDOMAIN);
                } elseif ($rules['basic_booking_period'] == 'day'){
                    $basic_booking_period_title = __('days', BABE_TEXTDOMAIN);
                }
                
                $price_types_array = self::get_price_types_from_rule($rules);
                $option_types_array = self::get_rate_types_from_rule($rules);
                
                if (self::base_rate_exist($post_id)){
                   unset($option_types_array['base']);
                }
                
                if (!empty($option_types_array)){
                
                    $output .= '<div id="rate_type_select">
                    <label for="_rate_type">'.__('Select rate type', BABE_TEXTDOMAIN).'</label>
                    <select name="_rate_type" id="_rate_type">';
                    
                    foreach($option_types_array as $value => $option_title){
                        $output .= '
                          <option value="'.$value.'">'.$option_title.'</option>
                        ';
                    }
 
                    $output .= '
                    </select>
                    
                    <button class="btn button button-primary" id="add_price">'.__('Save new rate', BABE_TEXTDOMAIN).'</button>
                    
                    </div>
                    
                    <div class="rate_title">
                        <input type="text" name="_rate_title" id="_rate_title" value="" placeholder="'.__('Enter rate title (required)', BABE_TEXTDOMAIN).'">
                    </div>
                    
                    <div class="rate_dates">
                       <input type="text" class="cmb2-text-small" name="_rate_date_from" id="_rate_date_from" value="" placeholder="'.__('date from', BABE_TEXTDOMAIN).'">
                       <input type="text" class="cmb2-text-small" name="_rate_date_to" id="_rate_date_to" value="" placeholder="'.__('date to', BABE_TEXTDOMAIN).'">
                    </div>
                    
                    ';
                    
                    foreach($price_types_array as $price_type=>$price_title){
                        
                        if ($price_type == 'weekly' && $basic_booking_period_title) {
                            $price_title_comment = sprintf(__('(price for 7 %s)', BABE_TEXTDOMAIN), $basic_booking_period_title);
                        } elseif ($price_type == 'monthly') {
                            $price_title_comment = __('(price for month)', BABE_TEXTDOMAIN);
                        } else {
                            $price_title_comment = '';
                        }
                        
                        
                    $output .= '<div class="set-price-block" data-type="'.$price_type.'" data-title="'.$price_title.'" data-title-alt="'.__('Flat rate', BABE_TEXTDOMAIN).'">
                    <h4>'.$price_title.' '.$price_title_comment.'</h4>';    
                    
                    $output .= '<table class="age-prices">
                    <tbody>';
                    
                    if ($price_type == 'weekend'){
                       $days_arr = BABE_Calendar_functions::get_week_days_arr();
                       $output .= '<ul class="cmb2-checkbox-list no-select-all cmb2-list">';
                       foreach ($days_arr as $day_num => $day_title){
                         $output .= $day_num < 1 || $day_num > 3 ? '<li><input type="checkbox" class="cmb2-option" name="weekend_days[]" id="weekend_days'.$day_num.'" value="'.$day_num.'"><label for="weekend_days'.$day_num.'">'.$day_title.'</label></li>' : '';
                       }
                       $output .= '</ul>';  
                    }
                    
                    if ($rules['ages']){
                    $ages = BABE_Post_types::get_ages_arr();
                    foreach ($ages as $age_id => $age_arr){
                       $output .= '<tr><td><span class="age_title">'. $age_arr['name'] . ' (' . $age_arr['description'] . ')</span> '.BABE_Currency::get_currency_symbol().' </td><td><input class="set-age-price age-price-'.$price_type.'" name="_price_'.$price_type.'['.$age_id.']" data-ind="'.$age_id.'" type="text" value=""></td></tr>'; 
                    }
                    } else {
                       $output .= '<tr><td><span class="age_title">'. __('Price', BABE_TEXTDOMAIN) . '</span> '.BABE_Currency::get_currency_symbol().' </td><td><input class="set-age-price age-price-'.$price_type.'" name="_price_'.$price_type.'[0]" data-ind="0" type="text" value=""></td></tr>'; 
                    }
                    $output .= '</tbody>
                    </table>
                    </div>';
                    
                    } //// end foreach $price_types_array
               } /// end if !empty($option_types_array)
             } // end if !empty($rules)
        }
        
        echo $output;
        wp_die();
        
    }
    
///////////////////////////////////////
    /**
	 * Get price types for current post's booking rules
     * @param array $rules
     * @return array
	 */
    public static function get_price_types_from_rule($rules){
        $output = array();
        
        $output['base'] = __('Price', BABE_TEXTDOMAIN);
                
        if ($rules['basic_booking_period'] == 'night' || $rules['basic_booking_period'] == 'day'){
                    $output['weekend'] = __('Weekend', BABE_TEXTDOMAIN);
                    $output['weekly'] = __('Weekly', BABE_TEXTDOMAIN);
                    $output['monthly'] = __('Monthly', BABE_TEXTDOMAIN);
                    $output['base'] = $rules['basic_booking_period'] == 'night' ? __('Nightly', BABE_TEXTDOMAIN) : __('Daily', BABE_TEXTDOMAIN);
        }
        
        return $output;
    }
///////////////////////////////////////
    /**
	 * Get rate types for current post's booking rules
     * @param array $rules
     * @return array
	 */
    public static function get_rate_types_from_rule($rules){
        $output = array();
        
        $output['base'] = __('Base', BABE_TEXTDOMAIN);
                
        if ($rules['basic_booking_period'] == 'night' || $rules['basic_booking_period'] == 'day'){
                    $output['seasonal'] = __('Seasonal', BABE_TEXTDOMAIN);
                  //  $output['event'] = __('Event', BABE_TEXTDOMAIN);
                  //// event is in the developement
        } elseif ($rules['basic_booking_period'] == 'recurrent_custom'){
                    $output['seasonal'] = __('Seasonal', BABE_TEXTDOMAIN);
        }
        
        return $output;
    }  
    
///////////////////////////////////////
    /**
	 * Get price types for current post's booking rules
     * @param array $rules
     * @return array
	 */
    public static function get_price_db_types_from_rule($rules){
        $output = array();
        
        $output['base'] = 'a_base';
                
        if ($rules['basic_booking_period'] == 'night' || $rules['basic_booking_period'] == 'day'){
                    $output['weekend'] = 'b_weekend';
                    $output['weekly'] = 'c_weekly';
                    $output['monthly'] = 'd_monthly';
        }
        
        return $output;
    }
///////////////////////////////////////
    /**
	 * Get rate types for current post's booking rules
     * @param array $rules
     * @return array
	 */
    public static function get_rate_db_types_from_rule($rules){
        $output = array();
        
        $output['base'] = 'a_base';
                
        if ($rules['basic_booking_period'] == 'night' || $rules['basic_booking_period'] == 'day'){
                    $output['seasonal'] = 'b_seasonal';
                    $output['event'] = 'c_event';
        } elseif ($rules['basic_booking_period'] == 'recurrent_custom'){
                    $output['seasonal'] = 'b_seasonal';
        }
        
        return $output;
    }         
        
///////////////////////////////////////    
    /**
	 * Create fields for price viewing.
	 */
    public static function ajax_get_price_details_block(){
        
        $output = '';
        
        if (isset($_POST['post_id']) && isset($_POST['cat_slug']) && isset($_POST['nonce']) && wp_verify_nonce( $_POST['nonce'], self::$nonce_title ) && BABE_Users::current_user_can_edit_post($_POST['post_id'])){
           $post_id = intval($_POST['post_id']);
           $cat_slug = sanitize_text_field($_POST['cat_slug']);
           $rules = BABE_Booking_Rules::get_rule_by_cat_slug($cat_slug);
           $ages = BABE_Post_types::get_ages_arr();
           $days_arr = BABE_Calendar_functions::get_week_days_arr();
             
           if (!empty($rules)){
            
            $rate_types = self::get_rate_types_from_rule($rules);
            $price_types = self::get_price_types_from_rule($rules);
        
            $rates = self::get_rates($post_id);
            
            if (!empty($rates)){
                
                foreach($rates as $rate){
                    if (isset($rate_types[$rate['rate_type']])){
                        $output .= '<div class="view-rate-block" data-rate-id="'.$rate['rate_id'].'">
                            <div class="view-rate-title opened" data-rate-id="'.$rate['rate_id'].'">'.$rate['rate_title'];
                            if ($rate['rate_type'] != 'base'){
                                $date_from = new DateTime($rate['date_from']);
                                $date_to = new DateTime($rate['date_to']);
                                $output .= '<div class="view-rate-dates">'.$date_from->format(get_option('date_format')).' - '.$date_to->format(get_option('date_format')).'</div>';
                            }
                            
                            $output .= '</div>
                            <div class="view-rate-details opened" data-rate-id="'.$rate['rate_id'].'">';     
                            
                       if ($rate['rate_type'] == 'base' || $rate['rate_type'] == 'seasonal'){
                          $price_types_tmp = $price_types;
                       } else {
                          $price_types_tmp['base'] = __('Flat rate', BABE_TEXTDOMAIN);
                       }
                            
                          foreach ($price_types_tmp as $price_type=>$price_title){
                            
                            if(isset($rate[$price_type]) && !empty($rate[$price_type])){
                            
                            //// get price block
                            $output .= '<div class="view-rate-details-item">
                            
                            <h4>'.$price_title.'</h4>';
                            
                            if ($price_type == 'weekend'){
                                $weekend_days = maybe_unserialize($rate['weekend_days']);
                                $output .= '<ul class="cmb2-checkbox-list no-select-all cmb2-list">';
                                foreach ($weekend_days as $key => $day_num){
                                    $output .= '<li><input type="checkbox" class="cmb2-option" name="weekend_days_'.$rate['rate_id'].'[]" id="weekend_days_'.$rate['rate_id'].'_'.$day_num.'" value="'.$day_num.'" checked="checked"><label for="weekend_days_'.$rate['rate_id'].'_'.$day_num.'">'.$days_arr[$day_num].'</label></li>';
                                }
                                $output .= '</ul>';  
                            }
                            $output .= '<table class="age-prices">
                            <tbody>';
                            
                            if (isset($rate[$price_type])){
                              $prices = $rate[$price_type];
                            
                              foreach($prices as $age_id => $price){
                                $age_title = !$age_id || !isset($ages[$age_id]) ? __('Price', BABE_TEXTDOMAIN) : $ages[$age_id]['name'] . ' (' . $ages[$age_id]['description'] . ')';
                                $output .= '<tr><td><span class="age_title">'. $age_title . '</span></td><td><div class="view-age-price">'.BABE_Currency::get_currency_price($price).'</div></td></tr>';
                              }
                            }
                            
                            $output .= '</tbody>
                            </table>
                            </div>'; // end view-rate-details-item
                            } //// end if
  
                          } ///  end foreach
                          
                       $output .= '<div class="view-rate-details-item-del" data-rate-id="'.$rate['rate_id'].'">
                           <i class="fas fa-trash-alt"></i></i>
                       </div>';   

                       $output .= '
                            </div>
                          </div>                
            '; //// end view-rate-block
                           
                    }
                } //////// end foreach rates
                    
             } // end if $rates   
           } // end if $rules  
         }
         
        echo $output;
        wp_die();  
    }
    
///////////ajax_check_base_rate////////
    /**
	 * Is there a base rate?
	 */
    public static function ajax_check_base_rate(){
        $output = '';
        
        if (isset($_POST['post_id']) && isset($_POST['nonce']) && wp_verify_nonce( $_POST['nonce'], self::$nonce_title ) && BABE_Users::current_user_can_edit_post($_POST['post_id'])){
             $post_id = intval($_POST['post_id']);
             $output = self::base_rate_exist($post_id) ? 1 : '';
        }     
        
        echo $output;
        wp_die();  
    }
    
//////////////ajax_delete_rate/////////    
    /**
	 * Delete selected rate.
	 */
    public static function ajax_delete_rate(){
        global $wpdb;
        $output = '';
        
        if (isset($_POST['post_id']) && isset($_POST['rate_id']) && isset($_POST['nonce']) && wp_verify_nonce( $_POST['nonce'], self::$nonce_title ) && BABE_Users::current_user_can_edit_post($_POST['post_id'])){
           $rate_id = intval($_POST['rate_id']); 
           $old_rate = $wpdb->get_row("SELECT * FROM ".self::$table_rate." WHERE rate_id = '".$rate_id."'", ARRAY_A);
       
           if(!empty($old_rate)){
           /// delete from DB
             $wpdb->query( $wpdb->prepare( 'DELETE FROM '.self::$table_rate.' WHERE rate_id = %d', $rate_id ) );
        
             $wpdb->query( $wpdb->prepare( 'DELETE FROM '.self::$table_rate_meta.' WHERE rate_id = %d', $rate_id ) );
           }
            
           $output = 1; 
        }
        
        echo $output;
        wp_die();  
    }
///////////////////////////////////////    
    /**
	 * Save rate.
	 */
    public static function ajax_save_rate(){
        global $wpdb;
        $output = '';
        
        if (isset($_POST['post_id']) && isset($_POST['cat_slug']) && isset($_POST['nonce']) && wp_verify_nonce( $_POST['nonce'], self::$nonce_title ) && BABE_Users::current_user_can_edit_post($_POST['post_id'])){
        
             $post_arr = self::sanitize_prices_post_arr();
             
             $rate_id = self::save_rate($post_arr);
        
        }
        
        echo $output;
        wp_die();                   
    }
    
///////////////////////////////////////    
    /**
	 * Save rate.
     * 
     * @param array $post_arr
     * 
     * @return int - rate ID
	 */
    public static function save_rate($post_arr){
        global $wpdb;
        $output = 0;
        
        //// _rate_type, _rate_title, _rate_date_from, _rate_date_to, 
        //// _price_base, _price_weekend, _price_weekly, _price_monthly, weekend_days
        //// post_id, cat_slug
        
        $rules = BABE_Booking_Rules::get_rule_by_cat_slug($post_arr['cat_slug']);
             
        if (!empty($rules) && $post_arr['post_id']){
                
             $price_types_array = self::get_price_db_types_from_rule($rules);
             $rate_types_array = self::get_rate_db_types_from_rule($rules);   
        
             $ins_arr = array(
                   'booking_obj_id' => $post_arr['post_id'],
                   'rate_type' => $rate_types_array[$post_arr['_rate_type']],
                   'rate_title' => $post_arr['_rate_title'],
                   'weekend_days' => maybe_serialize($post_arr['weekend_days']),
                   );
                if ($post_arr['_rate_date_from']){
                    $ins_arr['date_from'] = $post_arr['_rate_date_from'];
                } 
                if ($post_arr['_rate_date_to']){
                    $ins_arr['date_to'] = $post_arr['_rate_date_to'];
                }  
 
                //// create new row
                $wpdb->insert(
                  self::$table_rate,
                  $ins_arr
                 );
                 $rate_id = $wpdb->insert_id;
                 
                 if ($rate_id){
                    
                    $output = $rate_id;
                    
                    $divider = 1;
                    if ($post_arr['_rate_type'] == 'event'){
                        $rate_date_from = new DateTime( $post_arr['_rate_date_from'] );
                        $rate_date_to = new DateTime( $post_arr['_rate_date_to'] );
                        if ($rules['basic_booking_period'] == 'day'){
                            $rate_date_to->modify( '+1 day' );
                        }
                        $d_interval = date_diff($rate_date_from, $rate_date_to);
                        $days_total = $d_interval->format('%a');
                        $divider = $days_total;
                    }
                    //// add rate meta
                    foreach($price_types_array as $price_type => $price_db_type){
                        
                        foreach ($post_arr[$price_type] as $age_id => $price){
                            if ($price !== '' && floatval($price) >= 0){
                                $price = floatval($price);
                                $price = round($price, 2);
                                $c = 7;
                                $price_from = $price_type == 'weekly' ? round($price/$c, 2) : ($price_type != 'monthly' ? round($price/$divider, 2) : 0);
                                //// create new meta row
                                $wpdb->insert(
                                  self::$table_rate_meta,
                                  array(
                                    'rate_id' => $rate_id,
                                    'price_type' => $price_db_type,
                                    'age_id' => $age_id,
                                    'price' => $price,
                                    'price_from' => $price_from,
                                  )
                                );
                                /////db meta updated
                            }
                        }
                        
                    }  //// end foreach $price_types_array
             }  //// end if $rate_id
                 
        } /// end if !empty $rules
        
        return $output;                 
    }    
    
//////////////////////////////
    /**
	 * Sanitize prices POST array
     * @return array
	 */
    public static function sanitize_prices_post_arr(){
        
    $output = array();
    
    $output['post_id'] = isset($_POST['post_id']) && $_POST['post_id'] ? intval($_POST['post_id']) : 0;
    $output['cat_slug'] = isset($_POST['cat_slug']) && $_POST['cat_slug'] ? sanitize_text_field($_POST['cat_slug']) : '';
    $output['_rate_type'] = isset($_POST['_rate_type']) && $_POST['_rate_type'] ? sanitize_text_field($_POST['_rate_type']) : '';
    $output['_rate_title'] = isset($_POST['_rate_title']) && $_POST['_rate_title'] ? sanitize_text_field($_POST['_rate_title']) : '';
    
    $output['_rate_date_from'] = isset($_POST['_rate_date_from']) && BABE_Calendar_functions::isValidDate($_POST['_rate_date_from'], BABE_Settings::$settings['date_format']) ? BABE_Calendar_functions::date_to_sql($_POST['_rate_date_from']) : ''; /// now in Y-m-d format
    
    $output['_rate_date_to'] = isset($_POST['_rate_date_to']) && BABE_Calendar_functions::isValidDate($_POST['_rate_date_to'], BABE_Settings::$settings['date_format']) ? BABE_Calendar_functions::date_to_sql($_POST['_rate_date_to']) : ''; /// now in Y-m-d format
    
    if (isset($_POST['_price_base']) && !empty($_POST['_price_base'])){
        $output['base'] = $_POST['_price_base'];
    } else {
        $output['base'] = array();
    }
    
    if (isset($_POST['_price_weekend']) && !empty($_POST['_price_weekend'])){
        $output['weekend'] = $_POST['_price_weekend'];
    } else {
        $output['weekend'] = array();
    }
    
    if (isset($_POST['_price_weekly']) && !empty($_POST['_price_weekly'])){
        $output['weekly'] = $_POST['_price_weekly'];
    } else {
        $output['weekly'] = array();
    }
    
    if (isset($_POST['_price_monthly']) && !empty($_POST['_price_monthly'])){
        $output['monthly'] = $_POST['_price_monthly'];
    } else {
        $output['monthly'] = array();
    }
    
    if (isset($_POST['weekend_days']) && !empty($_POST['weekend_days'])){
        $output['weekend_days'] = array_map( 'intval', $_POST['weekend_days'] );
    } else {
        $output['weekend_days'] = array();
    }
    
    return $output;
    }

//////////////////////////////
//////////////////////////////
    /**
	 * Get total price clear and price with taxes by booking_obj_id from calculated $price_arr
     * 
     * @param int $booking_obj_id
     * @param array $price_arr
     * @return array
	 */
    public static function get_obj_total_price($booking_obj_id, $price_arr = array()){
        
        $price['total'] = 0;
        $price['total_with_taxes'] = 0;
        
        if (isset($price_arr['clear_with_taxes'])){
            foreach ($price_arr['clear'] as $item_price){
                $price['total'] += $item_price;
            }
            foreach ($price_arr['clear_with_taxes'] as $item_price){
                $price['total_with_taxes'] += $item_price;
            }
        }
        /// apply discount
        if ($price_arr['discount']){
            $multiplier = (100 - $price_arr['discount'])/100;
            $price['total'] = $price['total']*$multiplier;
            $price['total_with_taxes'] = $price['total_with_taxes']*$multiplier;
        }
        
        if (isset($price_arr['services'])){
            foreach ($price_arr['services'] as $service_prices){
                  foreach ($service_prices['clear'] as $item_price){
                    $price['total'] += $item_price;
                  }
                  foreach ($service_prices['clear_with_taxes'] as $item_price){
                    $price['total_with_taxes'] += $item_price;
                  }
            }
        }
        
        $price['deposit'] = $price_arr['deposit'];
        $price['total_deposit'] = round($price['total_with_taxes']*$price_arr['deposit']/100, 2);
        $price['payment_model'] = $price_arr['payment_model'];
        
        $price = apply_filters('babe_obj_total_price', $price, $booking_obj_id, $price_arr);
        
        return $price;
    }

//// extra place for places booking?

//////////////////////////////
    /**
	 * Get total prices array by booking_obj_id
     * @param int $booking_obj_id
     * @param string $date_from - format Y-m-d H:i
     * @param array $guests
     * @param string $date_to - format Y-m-d H:i
     * @param array $services
     * @return array
	 */
    public static function get_obj_total_price_arr($booking_obj_id, $date_from, $guests = array(), $date_to = '', $services = array()){
        
        $prices = array();
        
        $date_to = !$date_to ? $date_from : $date_to;
        
        $begin = new DateTime( $date_from );
        $end = new DateTime( $date_to );
        
        $begin_check = new DateTime( $begin->format('Y-m-d') );
        $end_check = new DateTime( $end->format('Y-m-d') );
        
        $rules_cat = BABE_Booking_Rules::get_rule_by_obj_id($booking_obj_id);
        
        $rates = self::get_rates($booking_obj_id, $begin->format('Y-m-d H:i:s'), $end->format('Y-m-d H:i:s'));
        
        $guests_for_obj = $rules_cat['rules']['ages'] ? $guests : array( 0 => array_sum($guests));
        
        if ($rules_cat['rules']['basic_booking_period'] == 'night' || $rules_cat['rules']['basic_booking_period'] == 'day'){
            //// get complex price
            
            if ($rules_cat['rules']['basic_booking_period'] == 'day' && $begin_check == $end_check){
              //  $end->modify( '+1 day' );
            }  
            
            $interval = new DateInterval('P1D');
            $daterange = new DatePeriod($begin, $interval, $end);
            foreach($daterange as $date){
              $date_rates_arr[$date->format("Y-m-d")] = 0; //// initial dates arr
            }
            
            //error_log('$rates: '.print_r($rates, 1));
            
            foreach ($rates as $rate){
                // workaround for base rate
                    $rate['date_from'] = $rate['date_from'] ? $rate['date_from'] : $date_from;
                    $rate['date_to'] = $rate['date_to'] ? $rate['date_to'] : $date_to;
                    //////
                    $rate_date_from = new DateTime( $rate['date_from'] );
                    $rate_date_to = new DateTime( $rate['date_to'] );
                    if (!($rules_cat['rules']['basic_booking_period'] == 'night' && $rate['rate_type'] == 'event')){
                      $rate_date_to->modify( '+1 day' );
                    } 
                    
                    $rate_begin_obj = $rate_date_from < $begin ? clone($begin) : clone($rate_date_from);
                    $rate_end_obj = $rate_date_to > $end ? clone($end) : clone($rate_date_to);
                    
                    $date_subrange = new DatePeriod($rate_begin_obj, $interval, $rate_end_obj);
                    
                    $rate_start = '';
                    $rate_end = '';
                    $last_date = '';
                    
                    foreach($date_subrange as $date){
                        if (!$date_rates_arr[$date->format("Y-m-d")]){
                            $rate_start = !$rate_start ? $date->format("Y-m-d") : $rate_start;
                            $date_rates_arr[$date->format("Y-m-d")] = $rate['rate_id'];
                        } elseif ($date_rates_arr[$date->format("Y-m-d")] != $rate['rate_id'] && $rate_start) {
                            $rate_end = $last_date;
                        }
                        if ($rate_start && $rate_end){
                            //// add prices
                            $prices = self::calculate_price($rate_start, $rate_end, $rate, $guests_for_obj, $rules_cat, $prices);
                            
                            $rate_start = '';
                            $rate_end = '';
                        }
                        $last_date = $date->format("Y-m-d");
                    }
                    if ($rate_start && !$rate_end){ 
                    //// add prices
                    $prices = self::calculate_price($rate_start, $last_date, $rate, $guests_for_obj, $rules_cat, $prices);
                    }
            }      
            
        } else {
            //// get base price foreach guest 
            $rate = $rates[1];
            $rate_start = $date_from;
            $rate_end = $date_to;
            $prices = self::calculate_price($rate_start, $rate_end, $rate, $guests_for_obj, $rules_cat, $prices);
        }
        
        foreach ($services as $service_id){
            $serv_tmp_prices = self::get_service_price($booking_obj_id, $service_id, $rules_cat, $date_from, $guests, $date_to, $prices);
            
            $prices['services'] = isset($prices['services']) ? $prices['services'] + $serv_tmp_prices['services'] : $serv_tmp_prices['services'];
        }
        
        //// get discount
        $discount_arr = BABE_Post_types::get_post_discount($booking_obj_id);
        $prices += $discount_arr;
        
        $deposit = apply_filters('babe_deposit_percents', $rules_cat['rules']['deposit'], $booking_obj_id, $rules_cat );
        
        $prices['deposit'] = $rules_cat['rules']['payment_model'] != 'full' && $deposit && $deposit <= 100 && $deposit > 0 ? $deposit : 100;
        
        $prices['payment_model'] = $rules_cat['rules']['payment_model'];
        
        return $prices;
        
    }
    
///////////////////////////////    
    /**
	 * Calculate price with selected rate
     * @param string $rate_start - format Y-m-d H:i
     * @param string $rate_end - format Y-m-d H:i
     * @param array $rate
     * @param array $guests
     * @param array $rules_cat
     * @param array $prices
     * @return array
	 */
    public static function calculate_price($rate_start, $rate_end, $rate, $guests, $rules_cat, $prices){
        
        //$prices_arr = array();
    
    if ($rate['rate_type'] == 'event' || $rules_cat['rules']['basic_booking_period'] == 'single_custom' || $rules_cat['rules']['basic_booking_period'] == 'recurrent_custom'){
        //// add prices by guest
        $prices = self::get_amount_by_price_type($rate, 'base', $guests, $rules_cat, $prices);
        
    } elseif ($rules_cat['rules']['basic_booking_period'] == 'month'){
        //// calculate number of months
        $rate_date_from = new DateTime( $rate_start );
        $rate_date_to = new DateTime( $rate_end );
        $d_interval = date_diff($rate_date_from, $rate_date_to);
        $months_total = $d_interval->format('%y') * 12 + $d_interval->format('%m');
        
        //// add prices by guest * months
        $prices = self::get_amount_by_price_type($rate, 'base', $guests, $rules_cat, $prices, $months_total);
        
    } else {
        $rate_date_from = new DateTime( $rate_start );
        $rate_date_to = new DateTime( $rate_end );
        
    //    if ($rules_cat['rules']['basic_booking_period'] == 'day')
             $rate_date_to->modify( '+1 day' );
                      
        $d_interval = date_diff($rate_date_from, $rate_date_to);
        //// split daterange by months, weeks, days
        if (!empty($rate['monthly']) && !empty($rate['weekly'])){
            $days_total = $d_interval->format('%a'); // total days
            $months_total = $d_interval->format('%y') * 12 + $d_interval->format('%m');
            $extra_weeks_total = intval(floor($d_interval->format('%d')/7));
            $extra_days_total = $d_interval->format('%d')%7;
        } elseif (!empty($rate['weekly'])) {
            $months_total = 0; 
            $days_total = $d_interval->format('%a'); // total days
            $extra_weeks_total = intval(floor($days_total/7));
            $extra_days_total = $days_total%7; 
        } else {
            $months_total = 0; 
            $days_total = $d_interval->format('%a'); // total days
            $extra_weeks_total = 0;
            $extra_days_total = $days_total;
        }
        
       if ($months_total){
        //// add prices by guest * number of span
       $prices = self::get_amount_by_price_type($rate, 'monthly', $guests, $rules_cat, $prices, $months_total); 
       }
    
       if ($extra_weeks_total){
        //// add prices by guest * number of span
        $prices = self::get_amount_by_price_type($rate, 'weekly', $guests, $rules_cat, $prices, $extra_weeks_total);
       }
    
       $weekend_days = 0;
    
      if ($extra_days_total){
       if (!empty($rate['weekend'])){
          $begin = clone($rate_date_to);
          $begin->modify( '-'.$extra_days_total.' day' );
          $interval = new DateInterval('P1D');
          $daterange = new DatePeriod($begin, $interval, $rate_date_to);
          foreach($daterange as $date){
            if (in_array(BABE_Calendar_functions::get_week_day_num($date), $rate['weekend_days'])){
                $weekend_days++;
            }  
          }
          //// add prices by guest * number of span
          $prices = self::get_amount_by_price_type($rate, 'weekend', $guests, $rules_cat, $prices, $weekend_days);
          // correct $extra_days_total
          $extra_days_total = $extra_days_total - $weekend_days; 
       }
       //// add prices by guest * number of span
       $prices = self::get_amount_by_price_type($rate, 'base', $guests, $rules_cat, $prices, $extra_days_total); 
      }  //// end $extra_days_total
        
    }  ///// end else
    
    return $prices;

   }
   
///////////////////////////////    
    /**
	 * Get amount by price type
     * 
     * @param array $rate
     * @param string $price_type
     * @param array $guests
     * @param array $rules_cat
     * @param array $prices
     * @param int $multiplier
     * @return array
	 */
    public static function get_amount_by_price_type($rate, $price_type, $guests, $rules_cat, $prices, $multiplier = 1){
        
        $prices_arr = array();

            $tax_am = isset($rules_cat['category_meta']['categories_add_taxes']) && $rules_cat['category_meta']['categories_add_taxes'] && isset($rules_cat['category_meta']['categories_tax']) ? floatval($rules_cat['category_meta']['categories_tax'])/100 : 0;
            // $tax = 1 + $tax_am;
        
        foreach ($guests as $age_id => $guests_number){
            $multiplier_local = $rules_cat['rules']['booking_mode'] != 'object' ? $multiplier*$guests_number : $multiplier;
            $prices_arr['clear'][$age_id] = isset($prices['clear'][$age_id]) ? $prices['clear'][$age_id] : 0;
            $price_clear = isset($rate[$price_type][$age_id]) ? $rate[$price_type][$age_id]*$multiplier_local : (isset($rate[$price_type][0]) ? $rate[$price_type][0]*$multiplier_local : 0);
            $prices_arr['clear'][$age_id] += $price_clear;

                $prices_arr['clear_with_taxes'][$age_id] = isset($prices['clear_with_taxes'][$age_id]) ? $prices['clear_with_taxes'][$age_id] : 0;
                $prices_arr['clear_with_taxes'][$age_id] += $price_clear + round($price_clear * $tax_am, 2);
        }
        
        return $prices_arr;
        
    }
    
///////////////////////////////    
    /**
	 * Get amount by service
     * 
     * @param int $booking_obj_id
     * @param int $service_id
     * @param array $rules_cat - rules with category_meta
     * @param string $date_from - format Y-m-d H:i
     * @param array $guests
     * @param string $date_to - format Y-m-d H:i
     * @param array $prices
     * @return array
	 */
    public static function get_service_price($booking_obj_id, $service_id, $rules_cat, $date_from, $guests = array(), $date_to = '', $prices = array()){
        
        $prices_arr = array();
        
        $date_to = !$date_to ? $date_from : $date_to;

        $tax_am = isset($rules_cat['category_meta']['categories_add_taxes']) && $rules_cat['category_meta']['categories_add_taxes'] && isset($rules_cat['category_meta']['categories_tax']) ? floatval($rules_cat['category_meta']['categories_tax'])/100 : 0;
        // $tax = 1 + $tax_am;
            
         /// get service meta
         $service_meta = (array)get_post_meta($service_id);
         foreach($service_meta as $key=>$val){
                $service_meta[$key] = maybe_unserialize($val[0]);
         }
         
         if (!isset($service_meta['prices'])){
            $service_meta['prices'][0] = 0;
         }
         
         if (isset($service_meta['service_type']) && isset($service_meta['price_type'])){
            $begin = new DateTime( $date_from );
            $end = new DateTime( $date_to );
            if ($service_meta['service_type'] == 'day' || $service_meta['service_type'] == 'person_day'){
              $end->modify( '+1 day' );
            }
            $d_interval = date_diff($begin, $end);
            $multiplier = $service_meta['service_type'] == 'person' || $service_meta['service_type'] == 'booking' ? 1 : $d_interval->format('%a'); // total days or nights
            
            if ($service_meta['service_type'] == 'booking' || $service_meta['service_type'] == 'day' || $service_meta['service_type'] == 'night'){  ///// per booking, per day, per night
                $price_clear = floatval($service_meta['prices'][0]);
                if ($service_meta['service_type'] == 'booking' && $service_meta['price_type'] == 'percent'){
                    $obj_total_price = self::get_obj_total_price($booking_obj_id, $prices);
                    $multiplier = $price_clear/100;
                    $prices_arr['services'][$service_id]['clear'][0] = $obj_total_price['clear']*$multiplier;
                    $prices_arr['services'][$service_id]['clear_with_taxes'][0] = round($obj_total_price['clear_with_taxes']*$multiplier, 2);
                } else {
                    $prices_arr['services'][$service_id]['clear'][0] = $price_clear*$multiplier;
                    $prices_arr['services'][$service_id]['clear_with_taxes'][0] = ($price_clear + round($price_clear * $tax_am, 2))*$multiplier;   
                }
            } else { ///// per person, per person_day, per person_night
                $tmp_price = 0;
                
                foreach ($guests as $age_id => $guests_number){
                    
                    if (isset($service_meta['prices'][$age_id])){
                        $price_clear = floatval($service_meta['prices'][$age_id]);
                        $tmp_price += $price_clear;
                        $prices_arr['services'][$service_id]['clear'][$age_id] = $price_clear*$guests_number*$multiplier;
                        $prices_arr['services'][$service_id]['clear_with_taxes'][$age_id] = ($price_clear + round($price_clear * $tax_am, 2))*$guests_number*$multiplier;                   
                    }
                } //// end foreach $guests
                if (!$tmp_price && !empty($guests)){
                    ///// we have $service_meta['prices'][0] only and $guests without 0 index 
                    //// get total guests
                    $guests_number = array_sum($guests);
                        $price_clear = floatval($service_meta['prices'][0]);
                        $tmp_price += $price_clear;
                        $prices_arr['services'][$service_id]['clear'][0] = $price_clear*$guests_number*$multiplier;
                        $prices_arr['services'][$service_id]['clear_with_taxes'][0] = ($price_clear + round($price_clear * $tax_am, 2))*$guests_number*$multiplier;
                }
            }  //// end if $service_meta['service_type'] == ...  
            
         } //// end if all service meta present
        
        return $prices_arr;
        
    }           

///////////////////////////////

    
}

BABE_Prices::init(); 
   