<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * BABE_shortcodes Class.
 * Shortcodes for the BA Book Everything plugin.
 * 
 * @class 		BABE_shortcodes
 * @version		1.2.0
 * @author 		Booking Algorithms
 */

BABE_shortcodes::init(); 

class BABE_shortcodes {
    
//////////////////////////////
    /**
	 * Hook in tabs.
	 */
    public static function init() {
        
        add_shortcode( 'all-items', array( __CLASS__, 'shortcode_all_items' ), 10 );
                
	}
    
//////////////////////////////

        /**
		 * Gets all items.
		 * 
		 * @param array $atts
		 * @param string $content
		 *
		 * @return string
		 */
		public static function shortcode_all_items( $atts, $content = null ) {
			
			$output = '';
            
            if ( class_exists( 'BABE_Post_types' ) ) {
			
			$args = shortcode_atts( array(
				'title'      => '',
				'ids'        => '',
                'category_ids' => '', //// term_taxonomy_ids from categories
				'per_page'   => 12,
				'fa_icon'    => '',
				'sort'       => 'rating',
				'sortby'     => 'DESC',
				'classes'    => '',
				'bg_img_url' => '',
			), $atts, 'all-items' );
			
			$post_args = array(
				'sort' => $args['sort'],
				'sort_by' => $args['sortby'],
				'posts_per_page' => $args['per_page'],
			);
			
			if ( $args['ids'] ) {
				$ids = explode(",", $args['ids']);
				$ids = array_map('intval', $ids);
				$ids = array_unique($ids);
				
				if ( ! empty( $ids ) ) {
					$post_args['post__in'] = $ids;
				}
			}
            
            if ( $args['category_ids'] ) {
				$category_ids = explode(",", $args['category_ids']);
				$category_ids = array_map('intval', $category_ids);
				$category_ids = array_unique($category_ids);
				
				if ( ! empty( $category_ids ) ) {
					$post_args['categories'] = $category_ids;
				}
			}
            
            $post_args = apply_filters('babe_shortcode_all_items_post_args', $post_args);
			
			$bg_style = $args['bg_img_url'] ? $bg_style = "style=\"background-image: url('" . esc_url( $args['bg_img_url'] ) . "');\"" : '' ;			
			
			$classes = $args['classes'] ? $args['classes'] : '';
			
			$fa_icon = $args['fa_icon'] ? '<i class="' . $args['fa_icon'] . '"></i>' : '';
			
			$content = $content ? '<div class="babe_shortcode_block_description">' . $content . '</div>' : '';   
			
			$output .= apply_filters('babe_shortcode_all_items_html', '
				<div class="babe_shortcode_block sc_all_items ' . $classes . '" ' . $bg_style . '>
					<div class="babe_shortcode_block_bg_inner">
						<h2 class="babe_shortcode_block_title">' . $fa_icon . $args['title'] . '</h2>
						' . $content . '
						<div class="babe_shortcode_block_inner">
							' . self::get_posts_tile_view( $post_args ) . '
						</div>
					</div>
				</div>
			', $args, $post_args);
			
            }
			
			return $output;
		}
        
//////////////////////////////
		/**
		 * Gets posts tile view.
		 * 
		 * @param array $post_args
		 * 
		 * @return string
		 */
		public static function get_posts_tile_view($post_args) {
		     
            $output = '';
             
            $posts = BABE_Post_types::get_posts( $post_args );
			
			$thumbnail = apply_filters('babe_shortcodes_all_item_thumbnail', 'ba-thumbnail');
            $excerpt_length = apply_filters('babe_shortcodes_all_item_excerpt_length', 13);
			
			foreach( $posts as $post ) {
             
             $image_srcs = wp_get_attachment_image_src( get_post_thumbnail_id( $post['ID'] ), $thumbnail );
				
			 $item_url = BABE_Functions::get_page_url_with_args($post['ID'], $_GET);
				
			 $image = $image_srcs ? '<a href="' . $item_url . '"><img src="' . $image_srcs[0] . '"></a>' : '';
				
			 $price_old = $post['discount_price_from'] < $post['price_from'] ? '<span class="item_info_price_old">' . BABE_Currency::get_currency_price( $post['price_from'] ) . '</span>' : '';
				
			 $discount = $post['discount'] ? '<div class="item_info_price_discount">-' . $post['discount'] . '%</div>' : '';
				
			 $babe_post = BABE_Post_types::get_post( $post['ID'] );
				
			 $output .= apply_filters('babe_shortcode_all_items_item_html', '
					<div class="babe_all_items_item">
						<div class="babe_all_items_item_inner">
							<div class="item_img">
								'.$image.'
							</div>
							<div class="item_text">
                                <div class="item_title">
                                   <a href="' . $item_url . '">' . apply_filters('translate_text', $post['post_title']) . '</a>
                                   ' . BABE_Rating::post_stars_rendering( $post['ID'] ) . '
                                </div>
								<div class="item_info_price">
									<label>' . __( 'from', BABE_TEXTDOMAIN ) . '</label>
									' . $price_old . '
									<span class="item_info_price_new">' . BABE_Currency::get_currency_price( $post['discount_price_from'] ) . '</span>
                                   ' . $discount . ' 
								</div>
								
								<div class="item_description">
									' . BABE_Post_types::get_post_excerpt( $post, $excerpt_length ) . '
								</div>
							</div>
						</div>
					</div>
				', $post, $babe_post);
            }    
            
            return $output;
             
		}        

//////////////////////////////    

}
